import sys; sys.path.append(".")
from models import *
from analog import *
from codelib import MatrixDefinition
import generate_code, re
from scipy.optimize import minimize, curve_fit
from scipy.signal import freqz

class Filter(object):

    def __init__(self, fs):
        self.fs = fs
        self.calc_coeffs()

    def freqz(self, R1, R5, C4, C5, P6v, w):
        args = (R1, R5, C4, C5, P6v)
        return freqz([v(*args) for v in self.b], [v(*args) for v in self.a], w)[1]

    def calc_coeffs(self):
        self.c = Circuit()
        S = ((R(5), 7, 8),
             (C(4), 7, 88),
             (R(1), 88, 9),
             (C(5), 8, 9),
             (P(6), GND, 8, 9),
             (IN, 7),
             (OUT, 9),
             )
        R1, R5, C4, C5, P6v = sp.symbols("R1,R5,C4,C5,P6v")
        V = {R(1): R1 * (1-P6v),
             R(5): R5 * (1-P6v),
             C(4): C4 * (1-P6v),
             C(5): C5 * (1-P6v),
             P(6): 1e6,
            }
        self.c.set_netlist(S, V)
        self.c._ensure_filter(symbolic=False)
        self.c.sim_filter.subst_var_default = dict(R1=1e4,R5=470e3,C4=470e-12,C5=1e-9,P6v=0.5)
        b, a = self.c.sim_filter.get_z_coeffs(samplerate=self.fs)
        args = (R1, R5, C4, C5, P6v)
        self.b = [sp.lambdify(args, v) for v in b]
        self.a = [sp.lambdify(args, v) for v in a]

    def get_coeffs(self, R1, R5, C4, C5):
        b, a = self.c.sim_filter.get_z_coeffs(as_expr=False)
        l = []
        for i, v in enumerate(b[0]+a[0]):
            s = set()
            for j, p in v:
                p = p.subs(dict(R1=R1,R5=R5,C4=C4,C5=C5))
                p = re.sub(r'([a-zA-Z0-9]+)\*\*(\d+)', r'pow(\1,\2)', str(p))
                j = j[0]
                l.append('    self.K%d(%d) = %s;' % (j, i, p))
                s.add(j)
            for j in sorted(set(range(max(s)+1))-s):
                l.append('    self.K%d(%d) = 0.0;' % (j, i))
        return l


class CompareFilter(object):

    def __init__(self):
        self.sig = Signal()
        self.c = Circuit()
        self.c.set_netlist(circ.Preamp_test.S, circ.Preamp_test.V)
        self.c.linearize()
        self.sr = self.sig(self.sig.sweep(), timespan=1)
        self.fr = (50,20000)
        self.f = numpy.logspace(numpy.log10(self.fr[0]), numpy.log10(self.fr[1]), 200)
        self.flt = Filter(self.c.FS)
        self.rg = np.logspace(0, -3, 6)
        self.c.set_pot_variable('P6v', 1.0)
        self.c.stream(self.sr)
        self.base = self.c.last_signal.get_spectrum(
            self.c.last_output, 2 * numpy.pi * self.f / self.c.FS)
        self.base = 20 * numpy.log10(abs(self.base))

    def get_coeffs(self, R1, R5, C4, C5):
        return self.flt.get_coeffs(R1, R5, C4, C5)

    def calc_highpass_f0(self, P6v, xdata, ydata):
        omega = numpy.pi * xdata / 96000
        p = [ 100e3, 470.e3,  470.e-12,  1.e-9]
        def f(w, R1, R5, C4, C5):
            return 20*numpy.log10(abs(self.flt.freqz(R1, R5, C4, C5, P6v, w)))
        p, cov = curve_fit(f, omega, ydata, p)
        return p

    def calc_param(self, rg):
        params = []
        for i, p in enumerate(rg):
            self.c.set_pot_variable('P6v', p)
            self.c.sim_c(numpy.zeros((self.c.FS,1)))
            self.c.stream(self.sr)
            h = self.c.last_signal.get_spectrum(
                self.c.last_output, 2 * numpy.pi * self.f / self.c.FS)
            h = 20 * numpy.log10(abs(h)) - self.base
            params.append(self.calc_highpass_f0(p, self.f, h))
        return params

    def plotcurves(self, param):
        for i, p in enumerate(self.rg):
            self.c.set_pot_variable('P6v', p)
            self.c.sim_c(numpy.zeros((self.c.FS,1)))
            lb = "%.1g" % p
            self.c.stream(self.sr)
            h = self.c.last_signal.get_spectrum(
                self.c.last_output, 2 * numpy.pi * self.f / self.c.FS)
            pylab.semilogx(self.f, 20*numpy.log10(abs(h))-self.base, label="c:"+lb)
            h = self.flt.freqz(*param, P6v=p, w=numpy.pi*self.f/self.c.FS)
            pylab.semilogx(self.f, 20*numpy.log10(abs(h)), label="f:"+lb)
        pylab.xlim(*self.fr)
        pylab.ylim(-60, 0)
        pylab.grid(which="both")
        pylab.legend(loc='lower right', ncol=2)
        pylab.show()


def calc_shape_transform(fnum, resnum, minmax, c):
    p0 = c.sim_py.p0.A1
    argstart = fnum*2
    if minmax[argstart+1,0] == minmax[argstart+1,1]:
        return numpy.array([[0.,0.,0.],[0.,0.,0.],[1.,0.,1.]])
    x = numpy.linspace(minmax[argstart,0], minmax[argstart,1], 200)
    p1 = float(p0[argstart+1])
    ap2 = numpy.linspace(minmax[argstart+1,0], minmax[argstart+1,1], 5)
    m0 = [1,0,1]
    r = numpy.zeros((len(ap2), 3))
    a = numpy.zeros((len(x),2))
    def f(x, p):
        a[:,0] = x
        a[:,1] = p
        return c.sim_c.c_calc_comp[fnum](a)
    for i, p2 in enumerate(ap2):
        def func(m):
            return numpy.sum(((m[0]*f(m[1]+m[2]*x,p1)) - f(x, p2)) ** 2)
        meth = "Powell"
        res = minimize(func, m0, method=meth)
        r[i] = res.x
        m0 = res.x
    return numpy.polyfit(ap2, r, 2)

def shape_transform_matrices(c):
    o = numpy.zeros((3, 3, 3))
    o[0] = calc_shape_transform(0, 0, c.minmax, c)
    o[1] = calc_shape_transform(1, 0, c.minmax, c)
    o[2] = calc_shape_transform(2, 0, c.minmax, c)
    l = []
    for i in range(o.shape[2]):
        for j in range(3):
            sn = "S%sm%d" % (['p','s','a'][j], i)
            l.append(MatrixDefinition(sn, array=True, value=o[:,j,i]))
    return l

class CodeGenerator(generate_code.CodeGenerator):
    def add_dict(self, d):
        self.pot_attr = self.pot_attr + [("P6v", "Volume", 5, False, None)]
        d = generate_code.CodeGenerator.add_dict(self, d)
        fl = CompareFilter()
        params = fl.calc_param(fl.rg[2:3])
        l = ["self.X.setZero();",
             "self.Y.setZero();",
             "unsigned int fs = samplingFreq;"]
        l.extend(fl.get_coeffs(*params[0]))
        d.overwrite("filter_init", "\n   ".join(l))
        fields = ["Array<double, %d, 1> %s;" % r for r in
                  ((6,"K0"),(6,"K1"),(6,"K2"),(6,"K3"),(6,"K4"),(6,"K5"),(6,"Y"),(6,"X"))]
        d.overwrite("DKPlugin_fields", "\n    ".join(fields))
        d.overwrite("add_npl", 1)
        d.overwrite("DKPlugin_init", ", K0(), K1(), K2(), K3(), K4(), K5(), Y(), X()")
        d.overwrite("process_add", """\
    for (int j = 0; j < n; j++) {
        self.pots_last[0] = 0.01 * t[0] + (1-0.01) * self.pots_last[0];
        double P6v = self.pots_last[0];
        Array<double, 6, 1> AB;
        AB = (((((self.K5 * P6v + self.K4) * P6v + self.K3) * P6v + self.K2) * P6v + self.K1) * P6v + self.K0);
        double out;
        out = ((u[j] * AB(0) + self.X(0) * AB(1) + self.X(1) * AB(2)) - (self.Y(0) * AB(4) + self.Y(1) * AB(5))) / AB(3);
        self.Y(1) = self.Y(0);
        self.Y(0) = out;
        self.X(1) = self.X(0);
        self.X(0) = u[j];
        o[j] = out;
    }
    u = o;
""")
        return d

# hand-optimized, unused, maybe out of date:
cc_func_code = """
    Array<creal, 2, 1> PP1;
    PP1 << pt(1), pt(5);
    Array<creal, 2, 1> PP0;
    PP0 << pt(0), pt(4);
    pt.head<2>() = (Spm1 * PP1 + Ssm1) * PP1 + Sam1 + ((Spm2 * PP1 + Ssm2) * PP1 + Sam2) * PP0;
    Array<creal, 6, 1> res;
    splinedata<unsigned short>::splev_pp<4>(&AmpData::nonlin_1::sd.sc[0], &pt(2), &res(2));
    splinedata<unsigned short>::splev_pp<4>(&AmpData::nonlin_1::sd.sc[1], &pt(2), &res(3));

    splinedata<unsigned short>::splev_pp<4>(&AmpData::nonlin_0::sd.sc[0], &pt(0), &res(0));
    splinedata<unsigned short>::splev_pp<4>(&AmpData::nonlin_0::sd.sc[1], &pt(0), &res(4));

    splinedata<unsigned short>::splev_pp<4>(&AmpData::nonlin_2::sd.sc[0], &pt(1), &res(1));
    splinedata<unsigned short>::splev_pp<4>(&AmpData::nonlin_2::sd.sc[1], &pt(1), &res(5));

    pt.head<2>() = ((Spm0 * PP1 + Ssm0) * PP1 + Sam0) * res.head<2>();
    pt.tail<2>() = ((Spm0 * PP1 + Ssm0) * PP1 + Sam0) * res.tail<2>();

    (*par.i).head<6>() << pt(0), pt(4), res(2), res(3), pt(1), pt(5), v[0], v[1];
"""

def highpass_as_postfilter(f0, FS):
    w0 = numpy.pi * f0 / FS
    b0 = 1 / (1 + w0)
    a1 = (1 - w0) * b0
    return (
        "{\n"
        "            static double y;\n"
        "            static double xm1;\n"
        "            y = %s * (buf[j] - xm1) + %s * y;\n"
        "            xm1 = buf[j];\n"
        "            buf[j] = y;\n"
        "        }\n"
        % (b0, a1))

numpy.set_printoptions(linewidth=2000)
sig = Signal()
sr = sig(20*sig.sine(freq=150), timespan=1)
s = sig(0.15*sig.sine(freq=150), timespan=0.2)
c = Circuit(FS=96000)
mod_id = "JCM800Pre"
c.set_module_id(mod_id)
c.plugindef = dk_simulator.PluginDef(mod_id)
c.plugindef.name = "JCM 800 Preamp"
c.plugindef.shortname = "JCM800Pre"
c.plugindef.description = "Simulation of JCM 800 preamp circuit"
c.plugindef.category = "Distortion"
c.plugindef.namespace = "jcm800pre"
c.plugindef.lv2_plugin_type = "SimulatorPlugin"
c.build_script = sys.argv[0]
c.set_netlist(circ.Preamp_test.S, circ.Preamp_test.V)
c.backward_euler = True
c.transform_opts.partition = True
c.transform_opts.decompose = True
c.sys_reduce_tol = 1e-5
#c.add_element(R('cc1'), (6, GND), 1e12)
#c.add_element(R('cc2'), (13, GND), 1e12)
c.add_element(R(61), (8, 9), 0.2e6),
c.add_element(R(62), (9, GND), 0.8e6),
c.remove_element("P6")
c.remove_element("Cm1")
#c.remove_element("Cm2")
#c.remove_element("Cm3")
c.build_verbose = True
c.set_tempdir("gencode")
c.keep_tempdir()
c.set_solver(dict(method='hybr', factor=1e5))
if 1:
    c.max_homotopy_iter = 10
    c.solver_params = [
        dict(method="hybr", factor=1e-1, max_homotopy_iter=10),
        dict(method="hybr", factor=1e-2, max_homotopy_iter=10),
        dict(method="hybr", factor=1e-1, max_homotopy_iter=10),
        ]
    c.basegrid = 6 * [ [[None, ('pp',4)], [None, None]], ]  # with shape transform
    c.stream(sig(0.0, samples=500))
    c.calc_range(sr)
    class CE(object):
        def __init__(self, atol, rtol):
            self.atol = atol
            self.rtol = rtol
        def __call__(self, f_intp, f_calc, f0):
            return abs(f_intp - f_calc) > self.atol + self.rtol * abs(f_calc - f0)
    c.E = [CE(2e-2, 5e-5), CE(1e-1, 5e-5), CE(1e-3, 5e-5),
           CE(1e-2, 5e-5), CE(1e-1, 5e-5), CE(1e-3, 5e-5)]
    c.max_homotopy_iter = 10
    c.set_use(SIM_TABLE)
    c._ensure_table_source()
    x0 = np.array([ -101.11169547,    -4.28127308,  -514.34759921,  -230.31382521,
                    -1133.65846006,    -4.20085902,   -20.3466745 ])
    c.sim_py.x0 = x0
    c.table_source["shape_transform"] = shape_transform_matrices(c)
    c.set_solver(dict(factor=1e5, xtol=1e-1, maxfev=12))
    c.post_filter = highpass_as_postfilter(20, c.FS)

c.code_generator = CodeGenerator
c.build_guitarix_module(include_paths=["../../../src/headers"])
#c.deploy()



import sys, os, traceback
import circuit
from tensbs import splinetable
import multiprocessing as mp
import numpy as np
from cStringIO import StringIO

def print_intpp_data(i):
    p = components_list[i]()
    o = StringIO()
    if hasattr(p, "init"):
        p.init()
    print >>o, "namespace %s {" % p.comp_id
    r, order_tab = splinetable.print_intpp_data(o, "", "", p, p.ranges, p.basegrid)
    print >>o, "splinecoeffs sc[%d] = {" % p.NVALS
    f_set = set()
    for j, row in enumerate(order_tab):
        inst = "splinedata::splev<%s>" % ",".join([str(v) for v in row if v is not None])
        f_set.add(inst)
        print >>o, "\t{x0_%d, xe_%d, hi_%d, n_%d, nmap_%d, map_%d, t_%d, c_%d, %s}," % (j, j, j, j, j, j, j, j, inst)
    print >>o, "};"
    print >>o, "splinedata sd = {"
    print >>o, "\tsc,"
    print >>o, "\t%d, /* number of calculated values */" % p.NVALS
    print >>o, "\t%d, /* number of input values */" % p.N_IN
    print >>o, "\t%d, /* number of output values */" % (p.NVALS-(p.NDIM-p.N_IN))
    print >>o, "\t%d, /* number of state values */" % (p.NDIM-p.N_IN)
    print >>o, '\t"%s",' % p.comp_id
    print >>o, "};"
    print >>o, "}; /* ! namespace %s */" % p.comp_id
    o.seek(0)
    return r, o.read(), f_set, p.comp_name, p.comp_id;

def print_header_file_start(h):
    print >>h, """\
#ifndef _DATA_H
#define _DATA_H 1

#include "intpp.h"

namespace AmpData {
    extern real b0;
    extern real b1;
    extern real a1;
    extern int fs;

"""

def print_header_file_end(h):
    print >>h, """\
}

#endif /* !_DATA_H */
"""

def print_header_file_entry(h, f):
    print >>h, "namespace %s { extern splinedata sd; }" % f

def print_header(o):
    print >>o, '#include "intpp.h"'
    print >>o, "namespace AmpData {"
    if 0:
        b0, b1, a1 = circuit.PhaseSplitter().feedback_coeff(1.0)
        print >>o, "real b0 = %g;" % b0
        print >>o, "real b1 = %g;" % b1
        print >>o, "real a1 = 1 - %g;" % (1 - a1)
    print >>o, "int fs = %d;" % circuit.FS
    return 3 * np.float32().nbytes + np.int32().nbytes

def print_footer(o):
    print >>o, "} // namespace AmpData"
    return 0

components_list = [
    circuit.PhaseSplitter_JCM800,
    circuit.PPGate_JCM800,
    circuit.PPPlate_JCM800,
    circuit.CoupledTriode,
    ]

def write_files(o, inst, h):
    if True: # set to False for sequential computation in the current process
        pool = mp.Pool(mp.cpu_count()+1)
        procs = [pool.apply_async(print_intpp_data, [i]) for i in range(len(components_list))]
    else:
        procs = [print_intpp_data(i) for i in range(len(components_list))]
    print_header_file_start(h)
    sz = print_header(o)
    l = []
    templ = set()
    for p in procs:
        if hasattr(p, "get"):
            p = p.get()
        s, f, i, comp_name, comp_id = p
        o.write(f)
        templ |= i
        l.append("%s: %d bytes" % (comp_name, s))
        sz += s
        print_header_file_entry(h, comp_id)
    sz += print_footer(o)
    l.append("data size sum: %d bytes" % sz)
    print >>o, "".join(["\n// " + s for s in l])
    print_header_file_end(h)
    for v in sorted(templ):
        print >>inst, "template int %s(splinecoeffs *p, real xi[2], real *res);" % v

def main():
    if len(sys.argv) > 1:
        o = open(sys.argv[1], "w")
        inst = open(sys.argv[2], "w")
        h = open(sys.argv[3], "w")
        files = sys.argv[1:4]
    else:
        o = sys.stdout
        inst = sys.stdout
        h = sys.stdout
        files = []
    try:
        write_files(o, inst, h)
    except Exception:
        traceback.print_exc()
        for f in files:
            try:
                os.remove(f)
            except:
                pass
        raise SystemExit(1)

if __name__ == "__main__":
    main()



from __future__ import division
import sys, os, re

def mksym(sym, nm, dev=None):
    if dev is None:
        dev = nm
    if sym.startswith(nm):
        sym = sym[len(nm):]
    try:
        int(sym)
    except ValueError:
        sym = "'%s'" % sym
    return "%s(%s)" % (dev, sym)

R_dict = {
    "k" : "e3",
    "M" : "e6",
    "." : "",
    ""  : "",
    }

C_dict = {
    "n" : "e-9",
    "p" : "e-12",
    "u" : "e-6",
    "." : "e-6",
    ""  : "e-6",
    }

L_dict = {
    "m" : "e-3",
    "u" : "e-6",
    "H" : "",
    "." : "",
    ""  : "",
    }

def resistor_value(val):
    m = re.match("(?P<l>[0-9]*)(\\.(?P<t>[0-9]*))?(?P<m>[kM])?$", val)
    if not m:
        m = re.match("(?P<l>[0-9]*)(?P<m>[kM.]?)(?P<t>[0-9]*)$", val)
    return m.group('l') + "." + (m.group('t') or '') + R_dict[m.group('m') or '']

def capacitor_value(val):
    m = re.match("(?P<l>[0-9]*)(\\.(?P<t>[0-9]*))?(?P<m>[npu])?F?$", val)
    if not m:
        m = re.match("(?P<l>[0-9]*)(?P<m>[npu.]?)(?P<t>[0-9]*)$", val)
    return m.group('l') + "." + (m.group('t') or '') + C_dict[m.group('m') or '']

mag_dict = {
    "M": "e6",
    "k": "e3",
    "m": "e-3",
    "u": "e-6",
    "n": "e-9",
    "p": "e-12",
    "f": "e-15",
    }

def convert_with_suffix(v):
    try:
        v = v[:-1] + mag_dict[v[-1]]
    except KeyError:
        pass
    return v

def Voltage(v):
    if v.endswith("V"):
        v = v[:-1]
    return convert_with_suffix(v)
    
def Current(v):
    if v.endswith("A"):
        v = v[:-1]
    return convert_with_suffix(v)
    
def Number(v):
    return convert_with_suffix(v)

def Text(v):
    return "'%s'" % v

def Resistance(v):
    return resistor_value(v)

def mk_dict(val, **kw):
    l = []
    for k, v in [v.split("=") for v in val.split(",")]:
        if k not in kw:
            print "warning: unknown parameter %s" % k
            try:
                v = float(v)
            except ValueError:
                pass
        else:
            v = kw[k](v)
        l.append("%s=%s" % (k, v))
    return "dict(%s)" % ", ".join(l)

def read_netlist(fname):
    cmd = "echo '(ampsim \"/dev/fd/3\")' | gnetlist -l ampsim.scm -i %s 3>&1 >/dev/null"
    rows = []
    values = {}
    with os.popen(cmd % fname) as f:
        out = []
        inp = []
        for line in f:
            line = line.split()
            sym = line[0]
            dev = line[1]
            val = line[2]
            conn = []
            for v in line[3:]:
                if v == "GND":
                    pass
                elif v.startswith("unnamed_net"):
                    v = "'u"+v[11:]+"'"
                elif v.startswith("unconnected_"):
                    v = None
                else:
                    try:
                        int(v)
                    except ValueError:
                        v = "'%s'" % v
                conn.append(v)
            if dev == "INPUT":
                inp += conn
                continue
            if dev == "OUTPUT":
                out += conn
                continue
            if dev in ("OPAMP","AOP-Standard"):
                conn = conn[:2] + conn[4:]
                sym = mksym(sym, "U", "OPA")
                #val = mk_dict(val,Vcc=Voltage,Vee=Voltage,A=Current)
                val = "Opamps['%s']" % val
            elif dev == "RESISTOR":
                sym = mksym(sym, "R")
                val = resistor_value(val)
            elif dev in ("CAPACITOR", "POLARIZED_CAPACITOR"):
                sym = mksym(sym, "C")
                val = capacitor_value(val)
            elif dev == "INDUCTOR":
                sym = mksym(sym, "L")
                m = re.match("([0-9]*)([muH.]?)([0-9]*)H?$", val)
                val = m.group(1)+"."+m.group(3)+L_dict[m.group(2)]
            elif dev == "TRANSFORMER":
                ##FIXME
                sym = mksym(sym, "Trans_L")
                conn = [conn[0], conn[4], conn[4], conn[1], conn[2], conn[3]]
                vl = {}
                d = {}
                for k, v in [v.split("=") for v in val.split(",")]:
                    if k == "R":
                        vl[k] = float(v)
                    elif k.startswith("w"):
                        d[int(k[1:])] = float(v)
                    else:
                        print "warning: unknown parameter %s" % k
                vl["windings"] = [v[1] for v in sorted(d.items())]
                val = repr(vl)
            elif dev == "NPN_TRANSISTOR":
                sym = mksym(sym, "T")
                if "=" in val:
                    val = mk_dict(val,Vt=Voltage,Is=Current,Bf=Number,Br=Number)
                else:
                    val = "Transistors['%s']" % val
            elif dev == "PNP_TRANSISTOR":
                ##FIXME
                sym = mksym(sym, "Tp")
                if "=" in val:
                    val = mk_dict(val,Vt=Voltage,Is=Current,Bf=Number,Br=Number)
                else:
                    val = "Transistors['%s']" % val
            elif dev == "DIODE":
                sym = mksym(sym, "D")
                if "=" in val:
                    ##FIXME
                    val = mk_dict(val,Is=Current,mUt=Number)
                    #val = mk_dict(val,Is=Current,Rs=Number,Bv=Number)
                    #Is=Saturation current=mA, RS=Ohmic resistance=Om, BV=Reverse breakdown voltage=V
                else:
                    val = "Diodes['%s']" % val
            elif dev == "DIODE2":
                sym = mksym(sym, "D2")
                if "=" in val:
                    ##FIXME
                    val = mk_dict(val,Is=Current,mUt=Number)
                    #val = mk_dict(val,Is=Current,Rs=Number,Bv=Number)
                    #Is=Saturation current=mA, RS=Ohmic resistance=Om, BV=Reverse breakdown voltage=V
                else:
                    val = "Diodes['%s']" % val
            elif dev == "TRIODE":
                sym = mksym(sym, "U", "Triode")
                val = "Tubes['%s']" % val
            elif dev == "PENTODE":
                sym = mksym(sym, "U", "Pentode")
                conn = [conn[0], conn[1], conn[3], conn[4]]
                val = "Tubes['%s']" % val
            elif dev == "VARIABLE_RESISTOR":
                sym = mksym(sym, "P")
                if "=" in val:
                    val = mk_dict(val,value=Resistance,a=Number,inv=Number,var=Text,name=Text)
                else:
                    val = resistor_value(val)
            elif dev == "CONNECTION_CURRENT":
                sym = mksym(sym, "CC")
                val = None
            elif dev == "POWER":
                sym = mksym(sym, "V")
                if val.endswith("V"):
                    val = val[:-1]
                if "." not in val:
                    val += "."
            elif dev == "VOLTAGE_SOURCE":
                sym = mksym(sym, "V")
                if val == "DC":
                    val = conn[0][1:-1]
                    conn = conn[1:]
                if val.endswith("V"):
                    val = val[:-1]
                if "." not in val:
                    val += "."
            elif dev == "vsin":
                continue
            else:
                assert False, dev
            if val is not None:
                values[sym] = val
            rows.append([sym]+conn)
        rows.append(['OUT']+out)
        rows.append(['IN']+inp)
    fmt_row = lambda row: " ".join(["None," if v is None else v+"," for v in row])
    return ("S = ((%s),\n     )" % "),\n     (".join([fmt_row(row) for row in rows]) +
            "\n" +
            "V = {%s}" % "".join(['%s: %s,\n     ' % v for v in sorted(values.items())])
            )
    return rows, values

if __name__ == "__main__":
    print read_netlist(sys.argv[1])



#! /usr/bin/env python
# encoding: utf-8
import sys
if sys.hexversion < 0x020400f0: from sets import Set as set
import os,sys,re
import TaskGen,Task,Utils,preproc,Logs,Build,Options
from Logs import error,debug,warn
from Utils import md5
from TaskGen import taskgen,after,before,feature
from Constants import*
from Configure import conftest
try:
	from cStringIO import StringIO
except ImportError:
	from io import StringIO
import config_c
USE_TOP_LEVEL=False
def get_cc_version(conf,cc,gcc=False,icc=False):
	cmd=cc+['-dM','-E','-']
	try:
		p=Utils.pproc.Popen(cmd,stdin=Utils.pproc.PIPE,stdout=Utils.pproc.PIPE,stderr=Utils.pproc.PIPE)
		p.stdin.write('\n')
		out=p.communicate()[0]
	except:
		conf.fatal('could not determine the compiler version %r'%cmd)
	out=str(out)
	if gcc:
		if out.find('__INTEL_COMPILER')>=0:
			conf.fatal('The intel compiler pretends to be gcc')
		if out.find('__GNUC__')<0:
			conf.fatal('Could not determine the compiler type')
	if icc and out.find('__INTEL_COMPILER')<0:
		conf.fatal('Not icc/icpc')
	k={}
	if icc or gcc:
		out=out.split('\n')
		import shlex
		for line in out:
			lst=shlex.split(line)
			if len(lst)>2:
				key=lst[1]
				val=lst[2]
				k[key]=val
		def isD(var):
			return var in k
		def isT(var):
			return var in k and k[var]!='0'
		mp1={'__linux__':'linux','__GNU__':'gnu','__FreeBSD__':'freebsd','__NetBSD__':'netbsd','__OpenBSD__':'openbsd','__sun':'sunos','__hpux':'hpux','__sgi':'irix','_AIX':'aix','__CYGWIN__':'cygwin','__MSYS__':'msys','_UWIN':'uwin','_WIN64':'win32','_WIN32':'win32',}
		for i in mp1:
			if isD(i):
				conf.env.DEST_OS=mp1[i]
				break
		else:
			if isD('__APPLE__')and isD('__MACH__'):
				conf.env.DEST_OS='darwin'
			elif isD('__unix__'):
				conf.env.DEST_OS='generic'
		if isD('__ELF__'):
			conf.env.DEST_BINFMT='elf'
		mp2={'__x86_64__':'x86_64','__i386__':'x86','__ia64__':'ia','__mips__':'mips','__sparc__':'sparc','__alpha__':'alpha','__arm__':'arm','__hppa__':'hppa','__powerpc__':'powerpc',}
		for i in mp2:
			if isD(i):
				conf.env.DEST_CPU=mp2[i]
				break
		debug('ccroot: dest platform: '+' '.join([conf.env[x]or'?'for x in('DEST_OS','DEST_BINFMT','DEST_CPU')]))
		conf.env['CC_VERSION']=(k['__GNUC__'],k['__GNUC_MINOR__'],k['__GNUC_PATCHLEVEL__'])
	return k
class DEBUG_LEVELS:
	ULTRADEBUG="ultradebug"
	DEBUG="debug"
	RELEASE="release"
	OPTIMIZED="optimized"
	CUSTOM="custom"
	ALL=[ULTRADEBUG,DEBUG,RELEASE,OPTIMIZED,CUSTOM]
def scan(self):
	debug('ccroot: _scan_preprocessor(self, node, env, path_lst)')
	if len(self.inputs)==1:
		node=self.inputs[0]
		(nodes,names)=preproc.get_deps(node,self.env,nodepaths=self.env['INC_PATHS'])
		if Logs.verbose:
			debug('deps: deps for %s: %r; unresolved %r',str(node),nodes,names)
		return(nodes,names)
	all_nodes=[]
	all_names=[]
	seen=set()
	for node in self.inputs:
		(nodes,names)=preproc.get_deps(node,self.env,nodepaths=self.env['INC_PATHS'])
		if Logs.verbose:
			debug('deps: deps for %s: %r; unresolved %r',str(node),nodes,names)
		for x in nodes:
			if id(x)in seen:continue
			seen.add(id(x))
			all_nodes.append(x)
		for x in names:
			if not x in all_names:
				all_names.append(x)
	return(all_nodes,all_names)
class ccroot_abstract(TaskGen.task_gen):
	def __init__(self,*k,**kw):
		if len(k)>1:
			k=list(k)
			if k[1][0]!='c':
				k[1]='c'+k[1]
		TaskGen.task_gen.__init__(self,*k,**kw)
def get_target_name(self):
	tp='program'
	for x in self.features:
		if x in['cshlib','cstaticlib']:
			tp=x.lstrip('c')
	pattern=self.env[tp+'_PATTERN']
	if not pattern:pattern='%s'
	dir,name=os.path.split(self.target)
	if self.env.DEST_BINFMT=='pe'and getattr(self,'vnum',None)and'cshlib'in self.features:
		name=name+'-'+self.vnum.split('.')[0]
	return os.path.join(dir,pattern%name)
def default_cc(self):
	Utils.def_attrs(self,includes='',defines='',rpaths='',uselib='',uselib_local='',add_objects='',p_flag_vars=[],p_type_vars=[],compiled_tasks=[],link_task=None)
	if not self.env.DEST_BINFMT:
		self.env.DEST_BINFMT=Utils.unversioned_sys_platform_to_binary_format(self.env.DEST_OS or Utils.unversioned_sys_platform())
	if not self.env.BINDIR:self.env.BINDIR=Utils.subst_vars('${PREFIX}/bin',self.env)
	if not self.env.LIBDIR:self.env.LIBDIR=Utils.subst_vars('${PREFIX}/lib${LIB_EXT}',self.env)
def apply_verif(self):
	if not(self.source or getattr(self,'add_objects',None)or getattr(self,'uselib_local',None)or getattr(self,'obj_files',None)):
		raise Utils.WafError('no source files specified for %s'%self)
	if not self.target:
		raise Utils.WafError('no target for %s'%self)
def vars_target_cprogram(self):
	self.default_install_path=self.env.BINDIR
	self.default_chmod=O755
def vars_target_cshlib(self):
	if self.env.DEST_BINFMT=='pe':
		self.default_chmod=O755
		self.default_install_path=self.env.BINDIR
	else:
		self.default_install_path=self.env.LIBDIR
def default_link_install(self):
	if self.install_path:
		self.bld.install_files(self.install_path,self.link_task.outputs[0],env=self.env,chmod=self.chmod)
def apply_incpaths(self):
	lst=[]
	for lib in self.to_list(self.uselib):
		for path in self.env['CPPPATH_'+lib]:
			if not path in lst:
				lst.append(path)
	if preproc.go_absolute:
		for path in preproc.standard_includes:
			if not path in lst:
				lst.append(path)
	for path in self.to_list(self.includes):
		if not path in lst:
			if preproc.go_absolute or not os.path.isabs(path):
				lst.append(path)
			else:
				self.env.prepend_value('CPPPATH',path)
	for path in lst:
		node=None
		if os.path.isabs(path):
			if preproc.go_absolute:
				node=self.bld.root.find_dir(path)
		elif path[0]=='#':
			node=self.bld.srcnode
			if len(path)>1:
				node=node.find_dir(path[1:])
		else:
			node=self.path.find_dir(path)
		if node:
			self.env.append_value('INC_PATHS',node)
	if USE_TOP_LEVEL:
		self.env.append_value('INC_PATHS',self.bld.srcnode)
def apply_type_vars(self):
	for x in self.features:
		if not x in['cprogram','cstaticlib','cshlib']:
			continue
		x=x.lstrip('c')
		st=self.env[x+'_USELIB']
		if st:self.uselib=self.uselib+' '+st
		for var in self.p_type_vars:
			compvar='%s_%s'%(x,var)
			value=self.env[compvar]
			if value:self.env.append_value(var,value)
def apply_link(self):
	link=getattr(self,'link',None)
	if not link:
		if'cstaticlib'in self.features:link='static_link'
		elif'cxx'in self.features:link='cxx_link'
		else:link='cc_link'
	tsk=self.create_task(link)
	outputs=[t.outputs[0]for t in self.compiled_tasks]
	tsk.set_inputs(outputs)
	tsk.set_outputs(self.path.find_or_declare(get_target_name(self)))
	self.link_task=tsk
def apply_lib_vars(self):
	env=self.env
	self.uselib=self.to_list(self.uselib)
	names=self.to_list(self.uselib_local)
	seen=set([])
	tmp=Utils.deque(names)
	while tmp:
		lib_name=tmp.popleft()
		if lib_name in seen:
			continue
		y=self.name_to_obj(lib_name)
		if not y:
			raise Utils.WafError('object %r was not found in uselib_local (required by %r)'%(lib_name,self.name))
		y.post()
		seen.add(lib_name)
		if getattr(y,'uselib_local',None):
			lst=y.to_list(y.uselib_local)
			if'cshlib'in y.features or'cprogram'in y.features:
				lst=[x for x in lst if not'cstaticlib'in self.name_to_obj(x).features]
			tmp.extend(lst)
		if getattr(y,'link_task',None):
			link_name=y.target[y.target.rfind(os.sep)+1:]
			if'cstaticlib'in y.features:
				env.append_value('STATICLIB',link_name)
			elif'cshlib'in y.features or'cprogram'in y.features:
				env.append_value('LIB',link_name)
			self.link_task.set_run_after(y.link_task)
			dep_nodes=getattr(self.link_task,'dep_nodes',[])
			self.link_task.dep_nodes=dep_nodes+y.link_task.outputs
			tmp_path=y.link_task.outputs[0].parent.bldpath(self.env)
			if not tmp_path in env['LIBPATH']:env.prepend_value('LIBPATH',tmp_path)
		for v in self.to_list(y.uselib):
			if not env['STATICLIB_'+v]:
				if not v in self.uselib:
					self.uselib.insert(0,v)
		if getattr(y,'export_incdirs',None):
			for x in self.to_list(y.export_incdirs):
				node=y.path.find_dir(x)
				if not node:
					raise Utils.WafError('object %r: invalid folder %r in export_incdirs'%(y.target,x))
				self.env.append_unique('INC_PATHS',node)
	for x in self.uselib:
		for v in self.p_flag_vars:
			val=self.env[v+'_'+x]
			if val:self.env.append_value(v,val)
def apply_objdeps(self):
	if not getattr(self,'add_objects',None):return
	seen=[]
	names=self.to_list(self.add_objects)
	while names:
		x=names[0]
		if x in seen:
			names=names[1:]
			continue
		y=self.name_to_obj(x)
		if not y:
			raise Utils.WafError('object %r was not found in uselib_local (required by add_objects %r)'%(x,self.name))
		if getattr(y,'add_objects',None):
			added=0
			lst=y.to_list(y.add_objects)
			lst.reverse()
			for u in lst:
				if u in seen:continue
				added=1
				names=[u]+names
			if added:continue
		y.post()
		seen.append(x)
		for t in y.compiled_tasks:
			self.link_task.inputs.extend(t.outputs)
def apply_obj_vars(self):
	v=self.env
	lib_st=v['LIB_ST']
	staticlib_st=v['STATICLIB_ST']
	libpath_st=v['LIBPATH_ST']
	staticlibpath_st=v['STATICLIBPATH_ST']
	rpath_st=v['RPATH_ST']
	app=v.append_unique
	if v['FULLSTATIC']:
		v.append_value('LINKFLAGS',v['FULLSTATIC_MARKER'])
	for i in v['RPATH']:
		if i and rpath_st:
			app('LINKFLAGS',rpath_st%i)
	for i in v['LIBPATH']:
		app('LINKFLAGS',libpath_st%i)
		app('LINKFLAGS',staticlibpath_st%i)
	if v['STATICLIB']:
		v.append_value('LINKFLAGS',v['STATICLIB_MARKER'])
		k=[(staticlib_st%i)for i in v['STATICLIB']]
		app('LINKFLAGS',k)
	if not v['FULLSTATIC']:
		if v['STATICLIB']or v['LIB']:
			v.append_value('LINKFLAGS',v['SHLIB_MARKER'])
	app('LINKFLAGS',[lib_st%i for i in v['LIB']])
def process_obj_files(self):
	if not hasattr(self,'obj_files'):return
	for x in self.obj_files:
		node=self.path.find_resource(x)
		self.link_task.inputs.append(node)
def add_obj_file(self,file):
	if not hasattr(self,'obj_files'):self.obj_files=[]
	if not'process_obj_files'in self.meths:self.meths.append('process_obj_files')
	self.obj_files.append(file)
c_attrs={'cxxflag':'CXXFLAGS','cflag':'CCFLAGS','ccflag':'CCFLAGS','linkflag':'LINKFLAGS','ldflag':'LINKFLAGS','lib':'LIB','libpath':'LIBPATH','staticlib':'STATICLIB','staticlibpath':'STATICLIBPATH','rpath':'RPATH','framework':'FRAMEWORK','frameworkpath':'FRAMEWORKPATH'}
def add_extra_flags(self):
	for x in self.__dict__.keys():
		y=x.lower()
		if y[-1]=='s':
			y=y[:-1]
		if c_attrs.get(y,None):
			self.env.append_unique(c_attrs[y],getattr(self,x))
def apply_implib(self):
	if not self.env.DEST_BINFMT=='pe':
		return
	self.meths.remove('default_link_install')
	bindir=self.install_path
	if not bindir:return
	dll=self.link_task.outputs[0]
	self.bld.install_files(bindir,dll,self.env,self.chmod)
	implib=self.env['implib_PATTERN']%os.path.split(self.target)[1]
	implib=dll.parent.find_or_declare(implib)
	self.link_task.outputs.append(implib)
	self.bld.install_as('${LIBDIR}/%s'%implib.name,implib,self.env)
	self.env.append_value('LINKFLAGS',(self.env['IMPLIB_ST']%implib.bldpath(self.env)).split())
def apply_vnum(self):
	if not getattr(self,'vnum','')or not'cshlib'in self.features or os.name!='posix'or self.env.DEST_BINFMT not in('elf','mac-o'):
		return
	self.meths.remove('default_link_install')
	link=self.link_task
	nums=self.vnum.split('.')
	node=link.outputs[0]
	libname=node.name
	if libname.endswith('.dylib'):
		name3=libname.replace('.dylib','.%s.dylib'%self.vnum)
		name2=libname.replace('.dylib','.%s.dylib'%nums[0])
	else:
		name3=libname+'.'+self.vnum
		name2=libname+'.'+nums[0]
	if self.env.SONAME_ST:
		v=self.env.SONAME_ST%name2
		self.env.append_value('LINKFLAGS',v.split())
	bld=self.bld
	nums=self.vnum.split('.')
	path=self.install_path
	if not path:return
	bld.install_as(path+os.sep+name3,node,env=self.env)
	bld.symlink_as(path+os.sep+name2,name3)
	bld.symlink_as(path+os.sep+libname,name3)
	tsk=self.create_task('vnum')
	tsk.set_inputs([node])
	tsk.set_outputs(node.parent.find_or_declare(name2))
def exec_vnum_link(self):
	path=self.outputs[0].abspath(self.env)
	try:
		os.remove(path)
	except OSError:
		pass
	try:
		os.symlink(self.inputs[0].name,path)
	except OSError:
		return 1
cls=Task.task_type_from_func('vnum',func=exec_vnum_link,ext_in='.bin',color='CYAN')
cls.quiet=1
def add_as_needed(conf):
	if conf.env.DEST_BINFMT=='elf'and'gcc'in(conf.env.CXX_NAME,conf.env.CC_NAME):
		conf.env.append_unique('LINKFLAGS','--as-needed')

feature('cc','cxx')(default_cc)
before('apply_core')(default_cc)
feature('cprogram','dprogram','cstaticlib','dstaticlib','cshlib','dshlib')(apply_verif)
feature('cprogram','dprogram')(vars_target_cprogram)
after('default_cc')(vars_target_cprogram)
before('apply_core')(vars_target_cprogram)
after('default_cc')(vars_target_cshlib)
feature('cshlib','dshlib')(vars_target_cshlib)
before('apply_core')(vars_target_cshlib)
feature('cprogram','dprogram','cstaticlib','dstaticlib','cshlib','dshlib')(default_link_install)
after('apply_link','vars_target_cprogram','vars_target_cshlib')(default_link_install)
feature('cc','cxx')(apply_incpaths)
after('apply_type_vars','apply_lib_vars','apply_core')(apply_incpaths)
feature('cc','cxx')(apply_type_vars)
after('init_cc','init_cxx')(apply_type_vars)
before('apply_lib_vars')(apply_type_vars)
feature('cprogram','cshlib','cstaticlib')(apply_link)
after('apply_core')(apply_link)
feature('cc','cxx')(apply_lib_vars)
after('apply_link','init_cc','init_cxx','apply_core')(apply_lib_vars)
feature('cprogram','cstaticlib','cshlib')(apply_objdeps)
after('init_cc','init_cxx','apply_link')(apply_objdeps)
feature('cprogram','cshlib','cstaticlib')(apply_obj_vars)
after('apply_lib_vars')(apply_obj_vars)
after('apply_link')(process_obj_files)
taskgen(add_obj_file)
feature('cc','cxx')(add_extra_flags)
before('init_cxx','init_cc')(add_extra_flags)
before('apply_lib_vars','apply_obj_vars','apply_incpaths','init_cc')(add_extra_flags)
feature('cshlib')(apply_implib)
after('apply_link','default_cc')(apply_implib)
before('apply_lib_vars','apply_objdeps','default_link_install')(apply_implib)
feature('cshlib')(apply_vnum)
after('apply_link')(apply_vnum)
before('apply_lib_vars','default_link_install')(apply_vnum)
conftest(add_as_needed)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2009, 2010 Hermann Meyer, James Warden, Andreas Degert
# Copyright (C) 2011 Pete Shorthose
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#----------------------------------------------------------------
#
# lots of hints and some numerical values taken from
# Virtual Air Guitar (Matti Karjalainen, Teemu Maki-Patola, Aki Kanerva, Antti Huovilainen)
#

import sys
from pylab import *
from scipy.optimize import newton

#
#                                 o V+         
#                                 |            
#                                 |            
#                                 |            
#                                +-+           
#                                | |           
#                             Rp | |           
#                                | |    Co     
#                                +-+           
#                                 |     ||       
#                                 +-----++---o Vo
#                              Vp |     ||
#                                 |
#     Ci                        --+--  
#                    Ri        /--+--  
#     ||  Vi      +-----+  Vg /         12AX7
# o---++--+-------|     |-----+- - - - 
#     ||  |       +-----+     \        
#         |                    \/----
#         |                    /-----
#         |                   /
#        +-+               Vk |
#        | |                  +------+    
#        | |                  |      |    
#        | |                  |      |    
#        +-+                 +-+     |    
#         |                  | |    -+-   
#         |               Rk | |    -+- Ck
#         |                  | |     |
#       -----                +-+     |
#        ---                  |      |
#         o                   |      |
#                             |      |
#                           -----  -----
#                            ---    ---
#                             o      o
#
#
# mu    Amplification factor.
# kx    Exponent:
# kg1   Inversely proportional to overall plate current
# kp    Affects opration in region of large plate voltage and large negative grid voltage
# kvb   Knee volts
#


names  = ("mu", "kx", "kg1", "kg2", "kp", "kvb", "ccg", "cpg", "ccp", "rgi")
factor = ( 1.0,  1.0,   1.0,   1.0,  1.0,   1.0, 1e-12, 1e-12, 1e-12,   1e3)
tubes  = {
    #TUBE           MU    EX      KG1     KG2     KP     KVB    CCG*  CPG*   CCP*  RGI**
    "12AX7"    : (100.0,  1.4,  1060.0,      0,  600.0, 300.0,  2.3,  2.4,   0.9,  2.0),
    "12AX7A"   : (100.3,  1.5,  1651.8,      0, 1795.7, 524.4,  2.3,  2.4,   0.9,  2.0),
    "12AX7ASYL": (105.8,  1.5,  1618.2,      0,  432.7,  35.6,  2.3,  2.4,   0.9,  2.0),
    "12AT7"    : ( 60.0,  1.35,  460.0,      0,  300.0, 300.0,  2.7,  2.2,   1.0,  2.0),
   #"12AT7"    : ( 67.5,  1.23,  419.1,      0,  214.0, 300.0,  2.3,  2.2,   1.0,  2.0),
   #"12AT7WA"  : ( 72.8,  1.67,  341.8,      0,  233.2, 267.8,  2.3,  2.2,   1.0,  2.0),
    "12AU7"    : ( 21.5,  1.3,  1180.0,      0,   84.0, 300.0,  2.3,  2.2,   1.0,  2.0),
   #"12AU7"    : ( 20.2,  1.23, 1108.0,      0,   84.9, 551.0,  2.3,  2.2,   1.0,  2.0),
    "12AY7"    : (44.16,  1.11, 1192.4,      0, 409.96, 300.0,  2.0,  2.0,   1.0,  2.0),
    "12AZ7"    : (74.08,  1.37,  382.4,      0, 190.11, 300.0,  2.3,  2.2,   1.0,  2.0),
    "12BH7A"   : (24.59,  1.34,  594.4,      0,  73.01, 300.0,  3.9,  3.2,   1.0,  2.0),
    "6AN8T"    : ( 21.5,   1.3, 1180.4,      0,  84.01, 300.0,  2.3,  2.2,   1.0,  2.0),
    "6DJ8"     : ( 28.0,  1.3,   330.0,      0,  320.0, 300.0,  2.3,  2.1,   0.7,  2.0),
    "6C16"     : ( 42.2,  2.21,  393.0,      0,  629.0, 446.0,  9.0,  1.8,  0.48,  2.0),
    "7025"     : (102.5,  1.41, 1598.8,      0,  813.8,  44.9,  2.3,  2.4,   0.9,  2.0),
    "SV6N1P"   : ( 36.6,  1.53, 1020.2,      0,  193.2, 300.0,  3.7,  2.3,   2.2,  2.0),
   #"2A3"      : ( 4.05,  1.63, 3652.2,      0,   58.5, 300.0, 17.0,  8.2,   6.0,  1.0),
   #"300B"     : ( 3.92,   1.5, 2140.3,      0,   64.0, 300.0, 17.0, 11.0,   7.5,  1.0),
   #"6C33C"    : (  3.1,   1.4,  163.0,      0,   15.0, 300.0, 31.0, 31.0,  11.0,  0.5),
    "ECL80T"   : ( 29.3,  1.33, 1443.9,      0,   45.5, 300.0,  3.2,  2.7,   1.8,  2.0),
    "ECL81T"   : ( 66.3,  1.39, 1400.2,      0,  369.3,   2.5,  3.2,  2.7,   1.8,  2.0),
    "ECL83T"   : (119.1,  1.32,  732.7,      0,  305.2,   1.8,  3.2,  2.7,   1.8,  2.0),
    "ECC83"    : ( 98.1,  1.46, 1734.7,      0,  754.4, 119.9,  2.3,  2.4,   0.9,  2.0),
    "ECC81"    : ( 77.1,  1.07,  320.1,      0,  231.0, 300.0,  2.3,  2.2,   1.0,  2.0),
    "JJECC81"  : ( 77.1,  1.08,  320.0,      0,  230.0, 300.0,  2.3,  2.2,   1.0,  2.0),
    "JJECC82"  : ( 24.1,  1.25, 1023.1,      0,   65.2, 300.0,  2.3,  2.2,   1.0,  2.0),
    "JJECC83S" : ( 98.3,  1.45, 1722.8,      0,  749.4, 131.2,  2.3,  2.4,   0.9,  2.0),
    "JJECC99"  : ( 23.1,  1.46,  455.3,      0,  191.9, 300.0,  2.3,  2.2,   1.0,  2.0),
    "JJEL34"   : ( 11.5,  1.35,  650.0, 6000.0,   51.8,  25.4, 15.0,  1.0,   8.0,  1.0),
    "JJKT88"   : ( 12.4,  1.21,  315.7, 4500.0,   26.7,  36.1, 14.0,  0.8,  12.0,  1.0),
    "JJ6L6GC"  : ( 13.9,  1.06,  320.2, 4500.0,   33.2,  29.6, 10.0,  0.6,   6.5,  1.0),
    "6550"     : (  7.9,  1.35,  890.0, 4800.0,   60.0,  24.0, 14.0, 0.85,  12.0,  1.0),
    "6AN8P"    : ( 11.0,  1.35,  650.0, 4200.0,   60.0,  24.0, 15.0,  1.0,   8.0,  1.0),
    "6L6CG"    : (  8.7,  1.35, 1460.0, 4500.0,   48.0,  12.0, 14.0,  0.85, 12.0,  1.0),
    "6L6GB"    : (  8.7,  1.26, 1210.3, 4500.0,   47.5,  11.6, 14.0,  0.85, 12.0,  1.0),
    "6AB8"     : ( 39.1,  0.93,  228.9, 4500.0,   50.2,  21.2,  6.7,  0.6,   4.1,  2.0),
    "6BQ5"     : ( 21.3,  1.24,  401.7, 4500.0,  111.0,  17.9, 10.0,  0.6,   5.1,  1.0),
    "7199P"    : ( 97.9,  1.35,  361.8, 4500.0,   79.6,  21.2,  5.7,  0.6,   2.5,  2.0),
    "KT88"     : (  8.8,  1.35,  730.0, 4200.0,   32.0,  16.0, 14.0, 0.85,  12.0,  1.0),
    "KT66"     : ( 11.7,  1.98,  510.9, 4500.0,   34.9,  22.3, 16.0,  2.3,  10.0,  1.0),
    "EL34_o"   : ( 11.0,  1.35,  650.0, 4200.0,   60.0,  24.0, 15.0,  1.0,   8.0,  1.0),
    "EL34"     : ( 12.3,  1.17,  353.9, 4500.0,   61.1,  29.9, 15.0,  1.0,   8.0,  1.0),
    "EL84"     : ( 21.3,  1.24,  401.7, 4500.0,  111.1,  17.9, 10.0,  0.6,   5.1,  1.0),
    "SVEL34"   : ( 11.5,  1.35,  608.9, 4500.0,   41.2,  30.1, 15.0,  1.0,   8.0,  1.0),
    "SV6L6GC"  : ( 11.7,  1.35,  996.4, 4500.0,   26.9,  22.1, 10.0,  0.6,   6.5,  1.0),
    "EF80"     : (49.11,  1.42,  667.0, 4500.0,  375.9,  42.9,  4.5,  0.6,   5.6,  2.0),
    "EF83"     : (12.01,  1.35, 2830.0, 4500.0,   35.2,  39.1,  4.5,  0.6,   5.6,  2.0),
    # * : 10^-12 (pF)
    # **: 10^3 (kOhm)
}

class Circuit(object):

    # class data
    used_names = ("mu", "kx", "kg1","kg2", "kp", "kvb", "Uin_min", "Uin_max", "Vp", "Rp")
    ipk_tab = { "triode": "Ipk_triode", "pentode": "Ipk_triode_pentode" , "pentode2": "Ipk_pentode"}

    @classmethod
    def help(self):
        return ("tube: %s\nplate current functions: %s" % (
            ", ".join(sorted(tubes.keys())),
            ", ".join(sorted(self.ipk_tab.keys()))))

    def __init__(self, tube, ipk_func):
        self.tube = tube
        self.ipk_func = ipk_func
        self.set_param()
        error = False
        if tube not in tubes:
            print "tube '%s' not found" % tube
            error = True
        if ipk_func not in self.ipk_tab:
            print "plate current function '%s' not found" % ipk_func
            error = True
        if error:
            print
            usage()
        for n, f, v in zip(names, factor, tubes[tube]):
            if v is not None:
                setattr(self, n, f*v)
        self.Ipk = getattr(self, self.ipk_tab[ipk_func])
        self.FtubeV = vectorize(self.Ftube)

    def set_param(self):
        # Parameters for circuit / approximation peer tube model
        if self.tube == "EL34" and self.ipk_func == "pentode":
            #self.Uin_range = (-20.0, 20.0)
            self.Uin_min   = -20
            self.Uin_max   = 20
            self.Vp        = 495
            self.Rp        = 3.5e3
        elif self.tube == "6L6CG" and self.ipk_func == "pentode":
            #self.Uin_range = ( -21, 21 )
            self.Uin_min   = -21
            self.Uin_max   = 21
            self.Vp        = 450
            self.Rp        = 5.0e3
        elif self.tube == "EL84" and self.ipk_func == "pentode":
            #self.Uin_range = ( -10, 10 )
            self.Uin_min   = -10
            self.Uin_max   = 10
            self.Vp        = 370
            self.Rp        = 3.5e3
        else: # triode tubes 
            #self.Uin_range = (-5.0, 5.0)
            self.Uin_min   = -5
            self.Uin_max   = 5
            self.Vp        = 250
            self.Rp        = 100e3
        self.table_size = 2001
        self.Ri_values = (68e3, 250e3)
        self.Vi = linspace(self.Uin_min,self.Uin_max,self.table_size)
        

    def Igk_Vgk(self, Vgk):
        """gate current as function of gate-kathode voltage"""
        return exp(7.75*Vgk-10.3)

    def Ipk_pentode(self, Vgk, Vpk):
        """
        E1 ={V(2,4)/KP*LOG(1+EXP((1/MU+V(3,4)/V(2,4))*KP))} ; E1 BREAKS UP LONG EQUATION FOR G1.
        G1 ={(PWR(V(7),EX)+PWRS(V(7),EX))/KG1*ATAN(V(1,4)/KVB)}
        G2 ={(EXP(EX*(LOG((V(2,4)/MU)+V(3,4)))))/KG2}
        """
        E1 = Vpk/self.kp*log(1+exp(self.kp*(1/self.mu+Vgk/Vpk)))
        G1 = 2*E1**self.kx/self.kg1*(E1>0.0)*arctan(Vpk/self.kvb)
        return (G1+exp(self.kx*(log(Vpk/self.mu)+Vgk)))/self.kg2 

    def Ipk_triode_pentode(self, Vgk, Vpk):
        """Koren model of pentode connected as class A triode
        (screen connected to plate):
        plate current as function of gate-kathode voltage
        and plate-kathode voltage
        """
        E1 = Vpk/self.kp*log(1+exp(self.kp*(1/self.mu+Vgk/Vpk)))
        return 2*E1**self.kx/self.kg1*(E1>0.0)*arctan(Vpk/self.kvb)

    def Ipk_triode(self, Vgk, Vpk):
        """
        Koren model of triode:
        plate current as function of gate-kathode voltage
        and plate-kathode voltage
        """
        E1 = Vpk/self.kp*log(1+exp(self.kp*(1/self.mu+Vgk/sqrt(self.kvb+Vpk*Vpk))))
        return 2*E1**self.kx/self.kg1*(E1>0.0)

    def Ftube(self, Vi, Ri):
        """calculate output voltage of a tube circuit as function of input voltage
        Vi  input voltage
        Ri  value of resistor Ri
        """
        def fi(Vgk, Vi, Ri):
            return Vi - Vgk - Ri * self.Igk_Vgk(Vgk) # sum of voltages -> zero

        Vgk = newton(fi, self.Igk_Vgk(0), args=(Vi, Ri))   # Vgk(Vi)

        def fp(Vpk, Vgk, Ipk):
            return Vpk + self.Rp * Ipk(Vgk, Vpk) - self.Vp

        return newton(fp, self.Vp/2, args=(Vgk,self.Ipk))   # Vpk(Vgk)

    def Vk0(self, Ri, Rk):
        v0 = 0
        def f(v):
            return (self.Ftube(-v, Ri)-self.Vp)*(Rk/self.Rp) + v
        return newton(f, v0)

    def write_ftube_table(self, Ri, Vi, Vp):
        """write C source"""
        sys.stdout.write("\t{ // Ri = %dk\n" % (Ri/1e3))
        sys.stdout.write('\t%g,%g,%g,%d, {' % (Vi[0], Vi[-1], (len(Vi)-1)/(Vi[-1]-Vi[0]), self.table_size))
        s = ""
        for i, v in enumerate(Vi):
            if i % 5 == 0:
                sys.stdout.write(s+"\n\t")
                s = ""
            sys.stdout.write(s+str(Vp[i]))
            s = ","
        sys.stdout.write("\n\t}}")

    def write_tables(self, prgname):
        sys.stdout.write("// generated by %s\n" % prgname)
        sys.stdout.write("// tube: %s\n" % self.tube)
        sys.stdout.write("// plate current function: %s\n" % self.ipk_func)
        for n in self.used_names:
            sys.stdout.write("// %s: %g\n" % (n, getattr(self, n)))
        sys.stdout.write("\n")
        sys.stdout.write("table1d_imp<%d> tubetable_%s[%d] __rt_data = {\n"
                         % (self.table_size, self.tube, len(self.Ri_values)))
        s = ""
        for Ri in self.Ri_values:
            sys.stdout.write(s)
            s = ",\n"
            Vp = self.FtubeV(self.Vi, Ri)
            self.write_ftube_table(Ri, self.Vi, Vp)
        sys.stdout.write("\n};\n")

    def R_name(self, r):
        for f, n in (1e6,"M"),(1e3,"k"),(1,""):
            if r >= f:
                return ("%g%s" % (r/f, n)).replace(".","_")

    def show_vk0(self, args):
        if args:
            Ri = float(args[0])
            Rk = float(args[1])
        else:
            try:
                while True:
                    vl = ["%d: %s" % (i, self.R_name(r)) for i, r in enumerate(self.Ri_values)]
                    i = raw_input("Ri [%s]: " % ", ".join(vl))
                    try:
                        i = int(i)
                    except ValueError:
                        pass
                    else:
                        if 0 <= i < len(self.Ri_values):
                            Ri = self.Ri_values[i]
                            break
                    print "error: illegal input"
                while True:
                    try:
                        Rk = float(raw_input("Rk: "))
                        break
                    except ValueError:
                        print "error: please enter float value"
            except KeyboardInterrupt:
                print
                return
        print "%f" % self.Vk0(Ri,Rk)

    def check_table_accuracy(self, Ri):
        """maximal relative table error at half interval"""
        def ip(x):
            return (x[:-1]+x[1:])/2
        Vp = self.FtubeV(self.Vi, Ri)
        VpM = self.FtubeV(ip(self.Vi), Ri)
        VpD = (ip(Vp) - VpM) / VpM
        return max(VpD)

    def display_accuracy(self):
        for Ri in self.Ri_values:
            print "Ri=%dk: %g" % (Ri/1e3, self.check_table_accuracy(Ri))
    
    def plot_Ftube(self):
        title(self.tube)
        for Ri in self.Ri_values:
            Vp = self.FtubeV(self.Vi, Ri)
            plot(self.Vi, Vp, label="Ri=%dk " % (Ri/1e3))
        xlabel("Vik")
        ylabel("Vp")
        legend()
        axis
        show()
    
    def s_plot_Ftube(self):
        title("%s-%s" % (self.tube, self.ipk_func))
        for Ri in self.Ri_values:
            Vp = self.FtubeV(self.Vi, Ri)
            plot(self.Vi, Vp, label="Ri=%dk \n Rk 2700 Vk0=%f\n Rk 1500 Vk0=%f\n Rk  820 Vk0=%f\n" % ((Ri/1e3), self.Vk0(Ri,2700.0), self.Vk0(Ri,1500.0), self.Vk0(Ri,820.0)))
        xlabel("Vik")
        ylabel("Vp")
        legend()
        axis
        savefig("%s.png" % self.tube)
        close()


def usage():
    print "usage: %s plot|s_plot|accuracy|table|vk0 tube-name plate-func" % sys.argv[0]
    print Circuit.help()
    raise SystemExit, 1

def main():
    if len(sys.argv) < 4:
        usage()
    cmd = sys.argv[1]
    c = Circuit(sys.argv[2], sys.argv[3])
    if cmd == "plot":
        c.plot_Ftube()
    elif cmd == "s_plot":
        c.s_plot_Ftube()
    elif cmd == "accuracy":
        c.display_accuracy()
    elif cmd == "table":
        c.write_tables(sys.argv[0])
    elif cmd == "vk0":
        c.show_vk0(sys.argv[4:])
    else:
        usage()

if __name__ == "__main__":
    main()



import numpy as np
cimport numpy as np

cdef extern from "ts9nonlin.h":
    double ts9nonlin(double x)
    extern float x0ts9 "ts9table.low"
    extern float hts9 "ts9table.istep"
    extern int nts9 "ts9table.size"

def ts9func(x):
    return ts9nonlin(x)

ts9func_range = [(x0ts9, hts9, nts9)]



import numpy as np
import ctypes as ct

__all__ = ["ab09ad","ab09ax","tb01id","tb01wd"]

def c_arr(w=False):
    flags = ['F']
    if w:
        flags.append('W')
    return np.ctypeslib.ndpointer(dtype=ct.c_double, ndim=1, flags=flags)
def c_mat(w=False):
    flags = ['F']
    if w:
        flags.append('W')
    return np.ctypeslib.ndpointer(dtype=ct.c_double, ndim=2, flags=flags)

slicot = ct.cdll.LoadLibrary("libslicot.so.0")

################################################################
## AB09AD
#
slicot.ab09ad_.restype = None
slicot.ab09ad_.argtypes = [
    ct.c_char_p, #DICO,
    ct.c_char_p, #JOB,
    ct.c_char_p, #EQUIL,
    ct.c_char_p, #ORDSEL,
    ct.POINTER(ct.c_int), #N,
    ct.POINTER(ct.c_int), #M,
    ct.POINTER(ct.c_int), #P,
    ct.POINTER(ct.c_int), #NR,
    c_mat(True), #A,
    ct.POINTER(ct.c_int), #LDA,
    c_mat(True), #B,
    ct.POINTER(ct.c_int), #LDB,
    c_mat(True), #C,
    ct.POINTER(ct.c_int), #LDC,
    c_arr(True), #HSV,
    ct.POINTER(ct.c_double), #TOL,
    np.ctypeslib.ndpointer(dtype=ct.c_int, ndim=1, flags=["F","W"]), #IWORK,
    c_arr(True), #DWORK,
    ct.POINTER(ct.c_int), #LDWORK,
    ct.POINTER(ct.c_int), #IWARN,
    ct.POINTER(ct.c_int), #INFO
    ]

def ab09ad(dico, job, equil, A, B, C, nr=None, tol=0):
    ordsel = 'A' if nr is None else 'F'
    Ar = np.array(A, dtype=np.float64, order='F', copy=True)
    Br = np.array(B, dtype=np.float64, order='F', copy=True)
    Cr = np.array(C, dtype=np.float64, order='F', copy=True)
    _nr = ct.c_int(nr or 0)
    hsv = np.zeros(Ar.shape[1], dtype=np.float64, order='F')
    iwork = np.zeros(Ar.shape[1], dtype=np.int32, order='F')
    dwork = np.zeros(Ar.size * 5, dtype=np.float64, order='F')
    iwarn = ct.c_int()
    info = ct.c_int()
    n = ct.c_int(Ar.shape[0])
    m = ct.c_int(Br.shape[1])
    p = ct.c_int(Cr.shape[0])
    lda = ct.c_int(Ar.shape[0])
    ldb = ct.c_int(Br.shape[0])
    ldc = ct.c_int(Cr.shape[0])
    _tol = ct.c_double(tol)
    ldwork = ct.c_int(dwork.shape[0])
    slicot.ab09ad_(
        dico, job, equil, ordsel, ct.byref(n), ct.byref(m), ct.byref(p),
        ct.byref(_nr), Ar, ct.byref(lda), Br, ct.byref(ldb), Cr, ct.byref(ldc),
        hsv, ct.byref(_tol), iwork, dwork, ct.byref(ldwork), ct.byref(iwarn), ct.byref(info))
    if iwarn.value != 0:
        print "ab09ad:", (
            "The selected order NR is greater than the order of a minimal\n"
            "realization of the given system. The order has been set according\n"
            "to the minimal realization of the system.")
    if info.value != 0:
        if info.value < 0:
            raise ValueError("%dth input argument is illegal" % -info.value())
        if info.value == 1:
            raise ValueError("the reduction of A to the real Schur form failed")
        if info.value == 2:
            raise ValueError("the state matrix A is not stable (if DICO = 'C') or not convergent (if DICO = 'D')")
        if info.value == 3:
            raise ValueError("the computation of Hankel singular values failed.")
        raise ValueError("unknown error value %d" % info.value)
    nr = _nr.value
    if nr != Ar.shape[1]:
        Ar = Ar[:nr,:nr]
        Br = Br[:nr]
        Cr = Cr[:,:nr]
    return Ar, Br, Cr, hsv

################################################################
## AB09AX
##
slicot.ab09ax_.restype = None
slicot.ab09ax_.argtypes = [
    ct.c_char_p, #DICO,
    ct.c_char_p, #JOB,
    ct.c_char_p, #ORDSEL,
    ct.POINTER(ct.c_int), #N,
    ct.POINTER(ct.c_int), #M,
    ct.POINTER(ct.c_int), #P,
    ct.POINTER(ct.c_int), #NR,
    c_mat(True), #A,
    ct.POINTER(ct.c_int), #LDA,
    c_mat(True), #B,
    ct.POINTER(ct.c_int), #LDB,
    c_mat(True), #C,
    ct.POINTER(ct.c_int), #LDC,
    c_arr(True), #HSV,
    c_mat(True), #T,
    ct.POINTER(ct.c_int), #LDT,
    c_mat(True), #TI,
    ct.POINTER(ct.c_int), #LDTI,
    ct.POINTER(ct.c_double), #TOL,
    np.ctypeslib.ndpointer(dtype=ct.c_int, ndim=1, flags=["F","W"]), #IWORK,
    c_arr(True), #DWORK,
    ct.POINTER(ct.c_int), #LDWORK,
    ct.POINTER(ct.c_int), #IWARN,
    ct.POINTER(ct.c_int), #INFO
    ]

def ab09ax(dico, job, A, B, C, nr=None, tol=0):
    ordsel = 'A' if nr is None else 'F'
    Ar = np.array(A, dtype=np.float64, order='F', copy=True)
    Br = np.array(B, dtype=np.float64, order='F', copy=True)
    Cr = np.array(C, dtype=np.float64, order='F', copy=True)
    _nr = ct.c_int(nr or 0)
    hsv = np.zeros(Ar.shape[1], dtype=np.float64, order='F')
    T = np.zeros_like(Ar)
    ldt = ct.c_int(Ar.shape[0])
    Ti = np.zeros_like(Ar)
    ldti = ct.c_int(Ar.shape[0])
    iwork = np.zeros(Ar.shape[1], dtype=np.int32, order='F')
    n = Ar.shape[0]
    m = Br.shape[1]
    p = Cr.shape[0]
    minsz = max(1,n*(max(n,m,p)+5) + n*(n+1)/2)
    dwork = np.zeros(minsz * 2, dtype=np.float64, order='F')
    iwarn = ct.c_int()
    info = ct.c_int()
    n = ct.c_int(Ar.shape[0])
    m = ct.c_int(Br.shape[1])
    p = ct.c_int(Cr.shape[0])
    lda = ct.c_int(Ar.shape[0])
    ldb = ct.c_int(Br.shape[0])
    ldc = ct.c_int(Cr.shape[0])
    _tol = ct.c_double(tol)
    ldwork = ct.c_int(dwork.shape[0])
    slicot.ab09ax_(
        dico, job, ordsel, ct.byref(n), ct.byref(m), ct.byref(p),
        ct.byref(_nr), Ar, ct.byref(lda), Br, ct.byref(ldb), Cr, ct.byref(ldc),
        hsv, T, ct.byref(ldt), Ti, ct.byref(ldti), ct.byref(_tol),
        iwork, dwork, ct.byref(ldwork), ct.byref(iwarn), ct.byref(info))
    if iwarn.value != 0:
        print "ab09ad:", (
            "The selected order NR is greater than the order of a minimal\n"
            "realization of the given system. The order has been set according\n"
            "to the minimal realization of the system.")
    if info.value != 0:
        if info.value < 0:
            raise ValueError("%dth input argument is illegal" % -info.value())
        if info.value == 1:
            raise ValueError("the state matrix A is not stable (if DICO = 'C') or not convergent (if DICO = 'D')")
        if info.value == 2:
            raise ValueError("the computation of Hankel singular values failed.")
        raise ValueError("unknown error value %d" % info.value)
    nr = _nr.value
    if nr != Ar.shape[1]:
        Ar = Ar[:nr,:nr]
        Br = Br[:nr]
        Cr = Cr[:,:nr]
        T = T[:,:nr]
        Ti = Ti[:nr,:]
    return T, Ti, Ar, Br, Cr, hsv

################################################################
## TB01ID
##
slicot.tb01id_.restype = None
slicot.tb01id_.argtypes = [
    ct.c_char_p, #JOB,
    ct.POINTER(ct.c_int), #N,
    ct.POINTER(ct.c_int), #M,
    ct.POINTER(ct.c_int), #P,
    ct.POINTER(ct.c_double), #MAXRED,
    c_mat(True), #A,
    ct.POINTER(ct.c_int), #LDA,
    c_mat(True), #B,
    ct.POINTER(ct.c_int), #LDB,
    c_mat(True), #C,
    ct.POINTER(ct.c_int), #LDC,
    c_arr(True), #SCALE,
    ct.POINTER(ct.c_int), #INFO
    ]

def tb01id(job, A, B, C, maxred=0):
    Ar = np.array(A, dtype=np.float64, order='F', copy=True)
    Br = np.array(B, dtype=np.float64, order='F', copy=True)
    Cr = np.array(C, dtype=np.float64, order='F', copy=True)
    n = ct.c_int(Ar.shape[0])
    m = ct.c_int(Br.shape[1])
    p = ct.c_int(Cr.shape[0])
    lda = ct.c_int(Ar.shape[0])
    ldb = ct.c_int(Br.shape[0])
    ldc = ct.c_int(Cr.shape[0])
    scale = np.zeros(Ar.shape[1], dtype=np.float64, order='F')
    info = ct.c_int()
    maxred_ = ct.c_double(maxred)
    slicot.tb01id_(job, ct.byref(n), ct.byref(m), ct.byref(p), ct.byref(maxred_),
                   Ar, ct.byref(lda), Br, ct.byref(ldb), Cr, ct.byref(ldc),
                   scale, ct.byref(info))
    if info.value != 0:
        if info.value < 0:
            raise ValueError("%dth input argument is illegal" % -info.value())
        raise ValueError("unknown error value %d" % info.value)
    return Ar, Br, Cr, maxred_.value, scale

################################################################
## TB01WD
##
slicot.tb01wd_.restype = None
slicot.tb01wd_.argtypes = [
    ct.POINTER(ct.c_int), #N,
    ct.POINTER(ct.c_int), #M,
    ct.POINTER(ct.c_int), #P,
    c_mat(True), #A,
    ct.POINTER(ct.c_int), #LDA,
    c_mat(True), #B,
    ct.POINTER(ct.c_int), #LDB,
    c_mat(True), #C,
    ct.POINTER(ct.c_int), #LDC,
    c_mat(True), #U,
    ct.POINTER(ct.c_int), #LDU,
    c_arr(True), #WR,
    c_arr(True), #WI,
    c_arr(True), #DWORK,
    ct.POINTER(ct.c_int), #LDWORK,
    ct.POINTER(ct.c_int), #INFO
    ]

def tb01wd(A, B, C):
    Ar = np.array(A, dtype=np.float64, order='F', copy=True)
    Br = np.array(B, dtype=np.float64, order='F', copy=True)
    Cr = np.array(C, dtype=np.float64, order='F', copy=True)
    n = ct.c_int(Ar.shape[0])
    m = ct.c_int(Br.shape[1])
    p = ct.c_int(Cr.shape[0])
    lda = ct.c_int(Ar.shape[0])
    ldb = ct.c_int(Br.shape[0])
    ldc = ct.c_int(Cr.shape[0])
    U = np.zeros_like(Ar)
    ldu = ct.c_int(Ar.shape[0])
    WR = np.zeros(Ar.shape[0], dtype=np.float64, order='F')
    WI = np.zeros(Ar.shape[0], dtype=np.float64, order='F')
    dwork = np.zeros(Ar.shape[0] * 5, dtype=np.float64, order='F')
    ldwork = ct.c_int(dwork.shape[0])
    info = ct.c_int()
    slicot.tb01wd_(
        ct.byref(n), ct.byref(m), ct.byref(p),
        Ar, ct.byref(lda), Br, ct.byref(ldb), Cr, ct.byref(ldc), U, ct.byref(ldu),
        WR, WI, dwork, ct.byref(ldwork), ct.byref(info))
    if info.value != 0:
        if info.value < 0:
            raise ValueError("%dth input argument is illegal" % -info.value())
        raise ValueError("not all eigenvalues computed (%d not computed)" % info.value)
    return Ar, Br, Cr, U, WR, WI



"""Support module for gtkparasite

This module contains functions for use in the gtkparasite console. It
uses some python magic to keep it compact (worth several thousand
lines of C code :-D)

usage:

start guitarix with gtkparasite:

$ GTK_MODULES=gtkparasite PYTHONPATH=<gx tool dir> ../build/default/src/gx_head/guitarix

Then shift the divider at the bottom of the gtkparasite window up to
enlarge the console and load this module:

>>> from psup import *

You can use the python help: e.g. help(psup) or help(pstyle) and of
course "pydoc psup" from the command line.

Select interesting widgets (with inspect and / or selecting from the
list) and use the context menu to copy them to the shell. If you type
a prefix like "w = " you will get the complete line assigning the
widget to the variable.

pstyle() prints style attributes of a widget while going up the widget
hierarchy. You can specify all x_XX attributes directly, xs_XX
attributes have to be called with a widget state constant (numeric or
symbolic).

diffstyle() prints attributes and values which differ between two
widgets. The last printed line is the command you can use with
copy-and-paste to view the differing attributes with pstyle(). You
need to change the widget name if it's not called w.

ls_style() print all registered style properties.

To reload the module if you did some changes while gtkparasite is
running:

>>> reload(psup); from psup import *
"""
from itertools import chain, izip
from pprint import pprint
import gtk
from gtk import (STATE_NORMAL, STATE_ACTIVE, STATE_PRELIGHT,
                 STATE_SELECTED, STATE_INSENSITIVE)
from gtk import widget_class_list_style_properties
import psup # self import for easy reload

def print_table(rows, enum=False, reverse=False):
    """print table of rows, formatted with optimal column width
    rows:    list of equal length rows
    enum:    prepend row number if True
    reverse: print list in reverse order
    """
    if not rows:
        return
    n = len(rows[0])
    fmt = " ".join(n * ["%-*s"])
    if enum:
        n += 1
        fmt = "%*s: " + fmt
    m = [0] * n
    lines = []
    for i, row in enumerate(rows):
        if enum:
            row.insert(0, str(i))
        r = []
        for j, v in enumerate(row):
            v = str(v)
            m[j] = max(m[j], len(v))
            r.append(v)
        lines.append(r)
    if reverse:
        lines.reverse()
    for row in lines:
        print fmt % tuple(chain.from_iterable(izip(m, row)))

def ls_style(widget):
    """print all style properties of widget"""
    l = []
    for p in widget_class_list_style_properties(widget):
        n = p.name
        v = widget.style_get_property(n)
        if isinstance(v, gtk.Border):
            v = "Border(left=%d,right=%d,top=%d,bottom=%d)" % (v.left, v.right, v.top, v.bottom)
        elif isinstance(v, str):
            v = repr(v)
        elif isinstance(v, float):
            v = "%g" % v
        l.append((n, v))
    print_table(l)

def pstyle(widget, *args):
    """tabulate a list of styles for the widget hierarchy
    use any number or "xs_XX(..)"- or "x_XX"-accessors as args after widget

    example: pstyle(w, xs_bg[1], xs_fg[STATE_NORMAL], x_black)

    state argument for color state accessor builders (xs_XX(state)):
    STATE_NORMAL      (0)
    STATE_ACTIVE      (1)
    STATE_PRELIGHT    (2)
    STATE_SELECTED    (3)
    STATE_INSENSITIVE (4)
    """
    l = []
    while True:
        s = widget.style
        l.append([widget.get_name()] + [f(s) for f in args])
        widget = widget.get_parent()
        if not widget:
            break
    last = None
    l2 = []
    for row in reversed(l):
        if last and row[1:] == last[1:]:
            s = ""
        else:
            s = "*"
        l2.append([s]+row)
        last = row
    l2.reverse()
    print_table(l2, enum=True, reverse=True)

def diffstyle(w1, w2):
    s1 = w1.style
    s2 = w2.style
    g = globals()
    d = []
    l = []
    def pdiff(f, n):
        v1 = f(s1)
        v2 = f(s2)
        if v1 == v2:
            return
        l.append((n, v1, v2))
        d.append(n)
    for v in sorted(g.keys()):
        if not v.startswith("x"):
            continue
        f = g[v]
        if v.startswith("xs"):
            for i in range(5):
                pdiff(f(i), "%s(%s)" % (v, i))
        else:
            pdiff(f, v)
    print_table(l)
    if l:
        print "pstyle(w,%s)" % ",".join(d)

for _v in "fg", "bg", "light", "dark", "mid", "text", "base", "text_aa":
    exec ("def xs_%(fld)s(state):\n"
          "    'field accessor builder (color state), use with pstyle'\n"
          "    def %(fld)s(s):\n"
          "        return s.%(fld)s[state]\n"
          "    return %(fld)s\n"
          ) % dict(fld=_v)

for _v in "black", "white", "xthickness", "ythickness":
    exec ("def x_%(fld)s(s):\n"
          "    'field accessor, use with pstyle'\n"
          "    return s.%(fld)s\n"
          ) % dict(fld=_v)

for _v in "family", "style", "variant", "weight", "stretch", "size", "size_is_absolute":
    exec ("def x_%(fld)s(s):\n"
          "    'field accessor, use with pstyle'\n"
          "    v = s.font_desc.get_%(fld)s()\n"
          "    try:\n"
          "        return v.value_nick\n"
          "    except AttributeError:\n"
          "        return v\n"
          ) % dict(fld=_v)
    

def x_font(s):
    "field accessor (pango font name), use with pstyle"
    return s.font_desc.to_string()



import numpy as np
cimport numpy as np
np.import_array()
from libc.string cimport const_char

cdef extern from "intpp.h":
    ctypedef float real
    struct splinecoeffs:
        real *x0
        real *xe
        real *stepi
        int *n
        int *nmap
        int **map
        real **t
        real **c
        int (*eval)(splinecoeffs *p, real *x, real *res)
    struct splinedata:
        splinecoeffs *sc
        int m
        int n_input
        int n_output
        int n_state
        const_char *func_id
    union retval "splinedata::retval":
        char c[4]
        int i

cdef extern from "data.h":
    splinedata ps_jcm800_sd "AmpData::ps_jcm800::sd"
    splinedata ppg_jcm800_sd "AmpData::ppg_jcm800::sd"
    splinedata ppp_jcm800_sd "AmpData::ppp_jcm800::sd"
    splinedata ct_sd "AmpData::ct::sd"

cdef rangeerror(int rv, real *Uin, splinedata *p, int n, int k):
    cdef retval cl
    cl.i = rv
    l = []
    for i in range(k):
        if not cl.c[i]:
            continue
        l.append("%g not in range [%g .. %g] (parameter %d)"
                 % (Uin[i], p.sc[n].x0[i], p.sc[n].xe[i], i+1))
    raise LookupError("%s/%d: %s" % (<char*>(p.func_id), n, " / ".join(l)))

def phasesplitter(Uin):
    assert len(Uin) == 4
    cdef real Uout[5]
    cdef real t[3]
    cdef int i
    t[0] = Uin[0] - Uin[1]
    t[1] = Uin[2]
    t[2] = Uin[3]
    cdef splinecoeffs *p
    cdef int rv
    for i in range(5):
        p = &ps_jcm800_sd.sc[i]
        rv = p.eval(p, t, &Uout[i])
        if rv != 0:
            rangeerror(rv, t, &ps_jcm800_sd, i, 4)
    for i in range(3):
        Uout[i+2] += Uin[i+1]
    return [v for v in Uout]

def pushpullgate(Uin):
    assert len(Uin) == 2
    cdef real Uout[2]
    cdef real t[2]
    cdef int i
    t[0] = Uin[0] - (Uin[1] - 311.045)
    cdef splinecoeffs *p
    cdef int rv
    for i in range(2):
        p = &ppg_jcm800_sd.sc[i]
        rv = p.eval(p, t, &Uout[i])
        if rv != 0:
            rangeerror(rv, t, &ppg_jcm800_sd, i, 1)
    Uout[1] += Uin[1] - 311.045
    return [v for v in Uout]

def pushpullplate(Uin):
    assert len(Uin) == 3
    cdef real Uout[2]
    cdef real t[3]
    cdef int i
    for i in range(3):
        t[i] = Uin[i]
    cdef splinecoeffs *p
    cdef int rv
    for i in range(2):
        p = &ppp_jcm800_sd.sc[i]
        rv = p.eval(p, t, &Uout[i])
        if rv != 0:
            rangeerror(rv, t, &ppp_jcm800_sd, i, 3)
    Uout[1] += t[2]
    return [v for v in Uout]

def coupledtriode(Uin):
    assert len(Uin) == 3
    cdef real Uout[3]
    cdef real t[3]
    cdef int i
    for i in range(3):
        t[i] = Uin[i]
    t[0] -= t[1]
    cdef splinecoeffs *p
    cdef int rv
    p = &ct_sd.sc[0]
    rv = p.eval(p, t, &Uout[0])
    if rv != 0:
        rangeerror(rv, t, &ct_sd, 0, 3)
    p = &ct_sd.sc[1]
    rv = p.eval(p, t, &Uout[1])
    if rv != 0:
        rangeerror(rv, t, &ct_sd, 1, 3)
    p = &ct_sd.sc[2]
    rv = p.eval(p, t, &Uout[2])
    if rv != 0:
        rangeerror(rv, t, &ct_sd, 2, 3)
    Uout[1] += t[1]
    Uout[2] += t[2]
    return [v for v in Uout]



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013, 2010 Hermann Meyer, Andreas Degert, Steve Poskitt
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#----------------------------------------------------------------
#


import sys, os, gtk, StringIO, glib, multiprocessing
from pylab import *
from time import sleep
from tube_transfer import *

class tube_transfer(gtk.Window):
    
    def on_exit(self, widget):
        self.destroy()
    
    def on_clear(self, widget):
        self.progressbar.hide()
        self.image.hide()
        self.textview.hide()
        self.resize(550, 200)
        self.labelvk01.set_text("\n")
        self.labelaccuracy.set_text("\n")
    
    def set_tube(self):
        tubelist = sorted(tubes.keys())
        for tube in tubelist:
            self.tube.append_text(tube)
    
    def get_param(self, c):
        self.table_size = c.table_size
        self.Ri_values = c.Ri_values
        self.scale6.set_value(c.Ri_values[0]/1e3)
        self.scale7.set_value(c.Ri_values[1]/1e3)
        self.scale1.set_value(c.Vp)
        self.scale2.set_value(c.Rp/1e3)
        self.scale3.set_value(c.Uin_min)
        self.scale4.set_value(c.Uin_max)
        
    
    def on_tube_changed(self,widget):
        c = Circuit(self.tube.get_active_text(), self.func.get_active_text())
        self.get_param(c)
        tubevalues = ""
        i = 0
        for n in c.used_names:
            tubevalues += " %s: %g " % (n, getattr(c, n))
            if i == 3:
                self.Kg2 = "%g" % (getattr(c, n))
            i +=1
            if i>5:
                 break
        self.labeltv.set_text(tubevalues)
        if self.Kg2 == "0":
            self.func.set_active(0)
        else:
            self.func.set_active(1)
    
    def on_type_changed(self,widget):
        c = Circuit(self.tube.get_active_text(), self.func.get_active_text())
        self.get_param(c)
        tubevalues = ""
        i = 0
        for n in c.used_names:
            tubevalues += " %s: %g " % (n, getattr(c, n))
            if i == 3:
                self.Kg2 = "%g" % (getattr(c, n))
            i +=1
            if i>5:
                 break
        self.labeltv.set_text(tubevalues)
        
    def set_func(self):
        self.func.append_text("triode")
        self.func.append_text("pentode")
        self.func.append_text("pentode2")
        self.func.set_active(0)
    
    def set_param(self, c):
        self.Ri_values = (self.scale6.get_value()*1e3, self.scale7.get_value()*1e3)
        self.table_size = c.table_size
        c.Ri_values    = self.Ri_values
        c.Vp           = self.scale1.get_value()
        c.Rp           = self.scale2.get_value()*1e3
        c.Uin_min      = self.scale3.get_value()
        c.Uin_max       = self.scale4.get_value()
        c.Vi = linspace(self.scale3.get_value(),self.scale4.get_value(),self.table_size)
    
    def on_accuracy(self, widget):
        c = Circuit(self.tube.get_active_text(), self.func.get_active_text())
        self.set_param(c)
        old_stdout = sys.stdout
        capturer = StringIO.StringIO()
        sys.stdout = capturer
        c.display_accuracy()
        sys.stdout = old_stdout
        output = capturer.getvalue() #.replace('\n','')
        output = os.linesep.join([s for s in output.splitlines() if s])
        self.labelaccuracy.set_text(output)
    
    def on_plot_progress(self):
        myfile="/tmp/tube.png"
        self.progressbar.show()
        if self.progressbar.get_fraction() >= 1.0:
            value = 0.0
        else:
            value = self.progressbar.get_fraction() + 0.1
        self.progressbar.set_fraction(value)
        if os.path.isfile(myfile):
            sleep(1)
            self.sw.set_size_request (-1, 480)
            self.image.set_from_file(myfile)
            self.textview.hide()
            self.image.show()
            glib.idle_add(self.on_plot_finish)
            self.progressbar.hide()
            return False
        return True

    def run_plot_progress(self):
        self.progressbar.set_text("plot %s" % self.tube.get_active_text())
        self.progressbar.set_fraction(0.0)
        glib.timeout_add(500, self.on_plot_progress)
        
    def on_plot_do(self):
        c = Circuit(self.tube.get_active_text(), self.func.get_active_text())
        self.set_param(c)
        myfile="/tmp/tube.png"
        if os.path.isfile(myfile):
            os.remove(myfile)
        title(self.tube.get_active_text())
        for Ri in c.Ri_values:
            Vp = c.FtubeV(c.Vi, Ri)
            plot(c.Vi, Vp, label="Ri=%dk " % (Ri/1e3))
        xlabel("Vik")
        ylabel("Vp")
        legend()
        axis
        savefig(myfile, dpi=80, bbox_inches=0)
        close()
        
    def on_plot(self, widget):
        jobs = []
        p = multiprocessing.Process(target=self.on_plot_do)
        jobs.append(p)
        p.start()
        self.run_plot_progress()
        
    def on_plot_finish(self):
        self.sw.set_size_request (-1, -1)
        
    def on_vk0(self, widget):
        c = Circuit(self.tube.get_active_text(), self.func.get_active_text())
        self.set_param(c)
        old_stdout = sys.stdout
        capturer = StringIO.StringIO()
        sys.stdout = capturer
        args = [0, self.scale5.get_value()*1e3]
        c.show_vk0(args)
        sys.stdout = old_stdout
        output = capturer.getvalue().replace('\n','')
        c = Circuit(self.tube.get_active_text(), self.func.get_active_text())
        self.set_param(c)
        old_stdout = sys.stdout
        capturer = StringIO.StringIO()
        sys.stdout = capturer
        args = [1, 1.5*1e3]
        c.show_vk0(args)
        sys.stdout = old_stdout
        output1 = capturer.getvalue().replace('\n','')
        c = Circuit(self.tube.get_active_text(), self.func.get_active_text())
        self.set_param(c)
        old_stdout = sys.stdout
        capturer = StringIO.StringIO()
        sys.stdout = capturer
        args = [1, 0.8*1e3]
        c.show_vk0(args)
        sys.stdout = old_stdout
        output2 = capturer.getvalue().replace('\n','')
        self.labelvk01.set_text("Ri(%s) Rk(%s) vk0(%s)  \nRi(%s) Rk(1.5) vk0(%s)   Ri(%s) Rk(0.8) vk0(%s)" % 
            (self.scale6.get_value(),self.scale5.get_value() ,output, self.scale7.get_value(), output1, self.scale7.get_value(), output2) )
    
    def on_table_progress(self):
        script_dir = sys.path[0]
        save_path = os.path.join(script_dir, 'gxtubes')
        myfile='%s/%s.cc' % (save_path,self.tube.get_active_text())
        self.progressbar.show()
        if self.progressbar.get_fraction() >= 1.0:
            value = 0.0
        else:
            value = self.progressbar.get_fraction() + 0.1
        self.progressbar.set_fraction(value)
        if os.path.isfile(myfile):
            sleep(1)
            infile = open(myfile, "r")
            if infile:
                self.sw.set_size_request (-1, 480)
                textbuffer = self.textview.get_buffer()
                string = infile.read()
                infile.close()
                textbuffer.set_text(string)
                self.image.hide()
                self.textview.show()
                glib.idle_add(self.on_plot_finish)
            self.progressbar.hide()
            
            return False
        return True
    
    def run_table_progress(self):
        self.progressbar.set_text("write table %s" % self.tube.get_active_text())
        self.progressbar.set_fraction(0.0)
        glib.timeout_add(500, self.on_table_progress)

    def on_table_do(self):
        script_dir = sys.path[0]
        save_path = os.path.join(script_dir, 'gxtubes')
        myfile='%s/%s.cc' % (save_path,self.tube.get_active_text())
        if not os.path.exists(save_path):
            os.makedirs(save_path)
        if os.path.isfile(myfile):
            os.remove(myfile)
        c = Circuit(self.tube.get_active_text(), self.func.get_active_text())
        self.set_param(c)
        old_stdout = sys.stdout
        capturer = StringIO.StringIO()
        sys.stdout = capturer
        c.write_tables(self.arg)
        sys.stdout = old_stdout
        output = capturer.getvalue()
        f = open(myfile, 'w')
        f.write(output)
        f.close

    def on_table(self, widget):
        jobs = []
        p = multiprocessing.Process(target=self.on_table_do)
        jobs.append(p)
        p.start()
        self.run_table_progress()
    
    def __init__(self, argv):        
        super(tube_transfer, self).__init__()
        self.arg = argv
        self.Kg2 = "0"
        self.set_default_size(550, 200)
        self.set_border_width(10)
        self.connect("destroy", lambda w: gtk.main_quit())
        self.set_title("Tube Calculator")
        vbox = gtk.VBox(False, 10)
        hbox = gtk.HBox(False, 8)
        hbox1 = gtk.HBox(False, 8)
        vbox1 = gtk.VBox(False, 10)
        self.sw = gtk.ScrolledWindow()
        self.sw.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        self.sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.progressbar = gtk.ProgressBar()
        table = gtk.Table(2,9,False)
        table.set_col_spacings(5)
        label = gtk.Label("Tube")
        self.tube = gtk.combo_box_new_text()
        self.set_tube()
        label1 = gtk.Label("Type")
        self.func = gtk.combo_box_new_text()
        self.set_func()
        self.func.connect("changed", self.on_type_changed)
        
        self.labelvk01 = gtk.Label("\n")
        self.labelvk01.set_selectable(True)
        self.labeltv = gtk.Label("")
        hbox1.pack_end(self.labeltv, False, False, 0)
        hbox1.pack_start(self.progressbar, False, False, 0)
        self.progressbar.hide()
        self.image = gtk.Image()
        self.textview = gtk.TextView()
        vbox1.pack_start(self.image, False, False, 0)
        vbox1.pack_start(self.textview, False, False, 0)
        self.sw.add_with_viewport(vbox1)
        
        label2 = gtk.Label("Rk (*e3)")
        self.scale5 = gtk.SpinButton()
        self.scale5.set_range(0, 1e3)
        self.scale5.set_increments(0.01, 10)
        self.scale5.set_value(2.7)
        self.scale5.set_digits(2)
        
        label3 = gtk.Label("Vp")
        self.scale1 = gtk.SpinButton()
        self.scale1.set_range(0, 1e3)
        self.scale1.set_increments(0.01, 10)
        self.scale1.set_value(250)
        self.scale1.set_digits(2)
        
        label4 = gtk.Label("Rp (*e3)")
        self.scale2 = gtk.SpinButton()
        self.scale2.set_range(0, 1e3)
        self.scale2.set_increments(0.01, 10)
        self.scale2.set_value(100)
        self.scale2.set_digits(2)
        
        label5 = gtk.Label("Uin_min")
        self.scale3 = gtk.SpinButton()
        self.scale3.set_range(-1e3, 0)
        self.scale3.set_increments(1, 10)
        self.scale3.set_value(-5)
        self.scale3.set_digits(0)
        
        label6 = gtk.Label("Uin_max")
        self.scale4 = gtk.SpinButton()
        self.scale4.set_range(0, 1e3)
        self.scale4.set_increments(1, 10)
        self.scale4.set_value(5)
        self.scale4.set_digits(0)
        
        label7 = gtk.Label("Ri[0] (*e3)")
        self.scale6 = gtk.SpinButton()
        self.scale6.set_range(0, 1e3)
        self.scale6.set_increments(1, 10)
        self.scale6.set_value(68)
        self.scale6.set_digits(0)
        
        label8 = gtk.Label("Ri[1] (*e3)")
        self.scale7 = gtk.SpinButton()
        self.scale7.set_range(0, 1e3)
        self.scale7.set_increments(1, 10)
        self.scale7.set_value(250)
        self.scale7.set_digits(0)
        
        fun = gtk.Button("plot")
        fun.connect("clicked", self.on_plot)
        fun1 = gtk.Button("table")
        fun1.connect("clicked", self.on_table)
        fun2 = gtk.Button("vk0")
        fun2.connect("clicked", self.on_vk0)
        self.labelaccuracy = gtk.Label("")
        fun3 = gtk.Button("accuracy")
        fun3.connect("clicked", self.on_accuracy)
        fun4 = gtk.Button("exit")
        fun4.connect("clicked", self.on_exit)
        fun5 = gtk.Button("clear")
        fun5.connect("clicked", self.on_clear)
        
        self.tube.connect("changed", self.on_tube_changed)
        self.tube.set_active(2)
        
        table.attach_defaults(label,0,1,0,1)
        table.attach_defaults(label1,1,2,0,1)
        table.attach_defaults(label2,2,3,0,1)
        table.attach_defaults(label3,3,4,0,1)
        table.attach_defaults(label4,4,5,0,1)
        table.attach_defaults(label5,5,6,0,1)
        table.attach_defaults(label6,6,7,0,1)
        table.attach_defaults(label7,7,8,0,1)
        table.attach_defaults(label8,8,9,0,1)
        
        table.attach_defaults(self.tube,0,1,1,2)
        table.attach_defaults(self.func,1,2,1,2)
        table.attach_defaults(self.scale5,2,3,1,2)
        table.attach_defaults(self.scale1,3,4,1,2)
        table.attach_defaults(self.scale2,4,5,1,2)
        table.attach_defaults(self.scale3,5,6,1,2)
        table.attach_defaults(self.scale4,6,7,1,2)
        table.attach_defaults(self.scale6,7,8,1,2)
        table.attach_defaults(self.scale7,8,9,1,2)
        
        vbox.pack_start(table, False, False, 0)
        vbox.pack_start(hbox1, False, False, 0)
        vbox.pack_start(self.sw, True, True, 0)
        vbox.pack_end(hbox, False, False, 0)
        
        hbox.pack_end(fun4, False, False, 0)
        hbox.pack_end(fun5, False, False, 0)
        hbox.pack_end(fun3, False, False, 0)
        hbox.pack_end(fun, False, False, 0)
        hbox.pack_end(fun1, False, False, 0)
        hbox.pack_end(fun2, False, False, 0)
        hbox.pack_end(self.labelvk01, False, False, 0)
        hbox.pack_start(self.labelaccuracy, False, False, 0)
        self.add(vbox)
        self.show_all()
        self.progressbar.hide()
        self.image.hide()
        self.textview.hide()
            
def main():
    tube_transfer(sys.argv[0])
    gtk.main()


if __name__ == "__main__":
    main()



import os, re, math, pickle
from pylab import *
from scipy.signal import freqz
from scipy import optimize

def display_parameter(filt):
    print "Parameter:"
    print "\n".join(["    %s=%s [%s..%s]" % ((k,filt[k])+filt.get_range(k))
                   for k in filt.keys()])

def plot_figure(filt, name, para, rg):
    display_parameter(filt)
    impulse = zeros(10000,dtype=float32)
    impulse[0] = 1
    dflt = filt[para]
    subplot(2,2,1)
    title('Guitarix crybaby2')
    ylabel("gain (dB)")
    xlabel("Frequency (Hz)")
    sn = False
    res = []
    for freq in rg:
        res.append(plotone(filt, freq, para, impulse, '-'))
    legend(title="Param.: %s" % name, fancybox=True, loc='upper right')
    a = array(res);
    subplot(2,2,2)
    plot(1/(1.06-a[:,0]), a[:,1])
    subplot(2,2,3)
    plot(a[:,0], a[:,2])
    subplot(2,2,4)
    plot(a[:,0], a[:,3])
    show()

def plot_figure_libcrybaby():
    from pluginloader import Plugin
    filt = Plugin("../build/default/src/plugins/libcrybaby.so")
    filt['crybaby2.refvolt'] = 0.1
    para = 'crybaby2.hotpotz'
    name = filt.get_var_attr(para)[0]
    rg = log10(linspace(*np.power(10,filt.get_range(para)), num=10))
    filt.init(48000)
    plot_figure(filt, name, para, rg)

def plot_figure_dunwah():
    import dunwah
    filt = dunwah.dsp()
    para = 'wah'
    name = "DunWah"
    rg = log10(linspace(*np.power(10,filt.get_range(para)), num=10))
    filt.init(48000)
    plot_figure(filt, name, para, rg)

def save_octave(name, var, fd):
    assert isinstance(var, ndarray) and len(var.shape) == 1
    print >>fd, "# name:", name
    iscompl = iscomplexobj(var)
    if iscompl:
        print >>fd, "# type: complex matrix"
    else:
        print >>fd, "# type: matrix"
    print >>fd, "# rows: 1"
    print >>fd, "# columns:", var.shape[0]
    if iscompl:
        for v in var:
            print >>fd, (v.real, v.imag),
    else:
        for v in var:
            print >>fd, v,
    print >>fd

def load_octave(fd):
    var = []
    d = {}
    for l in fd:
        if l.startswith("#"):
            m = re.match("# *([a-z]+): *(.*)\n", l)
            if not m:
                continue
            d[m.group(1)] = m.group(2)
        else:
            var.append(array([float(v) for v in l.split()]))
    return var

def invfreqz(w, h, numzeros=2, numpoles=2):
    fd = file("/tmp/out", "w");
    save_octave("F", w, fd)
    save_octave("H", h, fd)
    fd.close()
    p = os.popen("octave -q","w")
    p.write(
        "load /tmp/out;\n"
        "wt = 1 ./ F.^2;\n"
        "[B,A] = invfreqz(H,F,%d,%d,wt);\n"
        "save /tmp/out B A;\n"
        % (numzeros, numpoles))
    p.close()
    fd = file("/tmp/out");
    res = load_octave(fd)
    fd.close()
    os.remove("/tmp/out")
    return res

Bconst = array((1, -1.04, 0.04))

def estimate(rg, filt, para, impulse, fs):
    aa = zeros(len(rg))
    Q = zeros(len(rg))
    maxh = zeros(len(rg))
    for i, freq in enumerate(rg):
        filt[para] = freq
        filt.compute(zeros(10000,dtype=float32))
        w, h = freqz(filt.compute(impulse), worN=15000)
        k1 = int(round(len(h)*100.0/fs))
        k2 = int(round(len(h)*10000.0/fs))
        B, A = invfreqz(w[k1:k2+1], h[k1:k2+1])

        R = sqrt(A[2])
        theta = math.acos(A[1]/(-2*R))
        aa[i] = fs * theta
        frn = theta / (2*pi)
        Q[i] = (pi * frn) / (1 - R)

        print 'Pole frequency = %f Hz' % (aa[i]/(2*pi))
        #print 'Q = %f' % Q[i]
        #print "R =", R
        #print "frn =", frn, "theta =", theta

        A = array((1, -2*R*cos(theta), R*R))
        w1, h1 = freqz(Bconst, A, worN=15000)
        maxh[i] = max(abs(h))
        #print "gain =", gain[i]
    return aa, Q, maxh

def fit_param(rg, aa, Q, fs):
    def f(t):
        xx = 1/(t-aa)
        a = polyfit(rg, xx, 3)
        return sum((xx-polyval(a,rg))**2)
    off = optimize.fmin(f, 0.04*fs, disp=False)[0]
    f = 1/(off-aa)
    a1A = polyfit(rg, f, 3)
    #def ff(x, *p):
    #    return off - 1/polyval(p, x)
    #a1A = optimize.curve_fit(ff, rg, aa, a1A)[0]
    print "theta2pi = %g - 1 / (%s);" % (off, string_polyval(a1A, "wah"))
    qA = polyfit(rg, Q, 3)
    print "Q = %s;" % string_polyval(qA, "wah")
    return off, a1A, qA

def show_param(rg, off, g_off, aa, a1A, Q, qA, gain, gA):
    subplot(2,2,1)
    f = 1/(off-aa)
    plot(rg, f,"b")
    plot(rg,polyval(a1A,rg),"r")
    subplot(2,2,2)
    plot(rg, Q,"g")
    plot(rg, polyval(qA,rg), "r")
    subplot(2,2,3)
    plot(rg, 1/(g_off-gain), "b")
    plot(rg, polyval(gA,rg), "r")
    show()

def string_polyval(a, var):
    s = ""
    for i, v in enumerate(a):
        if i == 0:
            s = "%g" % v
        elif i == 1:
            s = "%s*%s+%g" % (s, var, v)
        else:
            s = "(%s)*%s+%g" % (s, var, v)
    return s

def make_gain(rg, off, a1A, qA, maxh, fs):
    gain = zeros(len(rg))
    for i, freq in enumerate(rg):
        q  = polyval(qA, freq)
        a1 = (off - 1 / polyval(a1A, freq)) / fs
        r = 1 - a1/(2*q)

        A = array((1, -2*r*math.cos(a1), r*r))
        w1, h1 = freqz(Bconst, A, worN=15000)
        gain[i] = maxh[i]/max(abs(h1))
    g_off = 0.025
    a = polyfit(rg, 1/(g_off-gain), 4)
    def f(x, *p):
        return g_off-1/polyval(p, x)
    a = optimize.curve_fit(f, rg, gain, a)
    a = a[0]
    print "g = %g - 1 / (%s):" % (g_off, string_polyval(a, "wah"))
    return g_off, gain, a

def output_freqz_libcrybaby():
    from pluginloader import Plugin
    filt = Plugin("../build/default/src/plugins/libcrybaby.so")
    filt['crybaby2.refvolt'] = 0.1
    para = 'crybaby2.hotpotz'
    name = filt.get_var_attr(para)[0]
    fs = 44100
    filt.init(fs)
    impulse = zeros(10000,dtype=float32)
    impulse[0] = 1
    #rg = linspace(*filt.get_range(para), num=20)
    rg = log10(linspace(*np.power(10,filt.get_range(para)), num=20))
    if True:
        aa, Q, maxh = estimate(rg, filt, para, impulse, fs)
        pickle.dump((aa,Q,maxh), file("p.out","w"))
    else:
        aa, Q, maxh = pickle.load(file("p.out"))
    off, a1A, qA = fit_param(rg, aa, Q, fs)
    g_off, gain, gA = make_gain(rg, off, a1A, qA, maxh, fs)
    show_param(rg, off, g_off, aa, a1A, Q, qA, gain, gA)
    rg = log10(linspace(*np.power(10,filt.get_range(para)), num=10))
    for i, freq in enumerate(rg):
        filt[para] = freq
        filt.compute(zeros(10000,dtype=float32))
        w, h = freqz(filt.compute(impulse), worN=15000)
        k1 = int(round(len(h)*30.0/fs))
        k2 = int(round(len(h)*5000.0/fs))
        B, A = invfreqz(w[k1:k2+1], h[k1:k2+1])

        R = sqrt(A[2])
        theta = math.acos(A[1]/(-2*R))
        aa[i] = A[1]/(-2*R)
        frn = theta / (2*pi)
        Q[i] = (pi * frn) / (1 - R)

        print 'Pole frequency = %f Hz' % (frn * fs)
        print 'Q = %f' % Q[i]

        if False:
            A = array((1, -2*R*cos(theta), R*R))
            w1, h1 = freqz(Bconst, A, worN=15000)
            gain[i] = max(abs(h))/max(abs(h1))
            B *= gain[i]
            h1 *= gain[i]
            print "gain =", gain[i]
            #h = h1

        if True:
            q  = polyval(qA, freq)
            a1 = (off - 1 / polyval(a1A, freq)) / fs
            frn = a1/(2*pi)
            r = 1 - a1/(2*q)
            #print "R =", r
            #print "frn =", frn, "theta =", math.acos(a1)
            g = g_off - 1 / polyval(gA, freq)

            A = array((1, -2*r*cos(a1), r*r))
            w1, h1 = freqz(Bconst*g, A, worN=15000)

            R = sqrt(A[2])
            theta = math.acos(A[1]/(-2*R))
            frn = theta / (2*pi)
            bq = (pi * frn) / (1 - R)
            print 'Pole frequency = %f Hz' % (frn * fs)
            print 'Q = %f' % q
            print "gain =", g

        w *= fs / (2*pi)
        semilogx(w[1:], 20*log10(abs(h[1:])), "b")
        semilogx(w[1:], 20*log10(abs(h1[1:])), "r")
    show()

def display_parameter(filt):
    print "Parameter:"
    print "\n".join(["    %s=%s [%s..%s]" % ((k,filt[k])+filt.get_range(k))
                   for k in filt.keys()])

if __name__ == "__main__":
    try:
        #plot_figure_libcrybaby()
        #plot_figure_dunwah()
        output_freqz_libcrybaby()
    except KeyboardInterrupt:
        pass



import sys; sys.path.append(".")
from models import *
from analog import *
from scipy.optimize import curve_fit

def calc_highpass_f0(c1, c2):
    sig = Signal()
    s = c1.make_signal_vector(sig(0.01*sig.impulse(), timespan=1))
    f0 = numpy.zeros(11)
    fl = numpy.logspace(numpy.log10(s.start_freq), numpy.log10(s.stop_freq), 200)
    w = 2 * numpy.pi * fl / s.fs
    for i, hotpotz in enumerate(numpy.linspace(0, 1, 11)):
        c1.set_pot_variable('hotpotz', hotpotz)
        c1.stream(s)
        h1 = s.get_spectrum(c1.last_output[:,0], w)

        c2.set_pot_variable('hotpotz', hotpotz)
        c2.stream(s)
        h2 = s.get_spectrum(c2.last_output[:,0], w)

        ydata = numpy.log(abs(h1/h2))
        e = numpy.exp(-1j*w)
        a1 = -1
        def f(e, a1):
            return numpy.log(abs((1-a1)/2 * (1 - e) / (1 + a1 * e)))
        res = curve_fit(f, e, ydata, a1)

        a1 = res[0][0]
        f0[i] = s.fs*(1 + a1)/(numpy.pi*(1 - a1))
    return numpy.mean(f0)

def highpass_as_prefilter(f0, FS):
    w0 = numpy.pi * f0 / FS
    b0 = 1 / (1 + w0)
    a1 = (1 - w0) * b0
    return (
        "static DTP_U y; {\n"
        "            static double xm1;\n"
        "            y = %s * (GET_U[0] - xm1) + %s * y;\n"
        "            xm1 = GET_U[0];\n"
        "        }\n"
        "#undef GET_U\n"
        "#define GET_U (&y)\n"
        % (b0, a1))

def build_module(FS, postfix, linear=False):
    c1 = Circuit(FS=FS)
    c1.build_script = sys.argv[0]
    c1.set_tempdir("gencode")
    c1.keep_tempdir()
    mod_id = "GCB_95%s" % postfix
    c1.set_module_id(mod_id)
    c1.plugindef = dk_simulator.PluginDef(mod_id)
    c1.plugindef.name = "GCB 95"
    c1.plugindef.description = "Linear filter simulating the GCB 95 crybaby circuit"
    c1.plugindef.category = "Guitar Effects"
    c1.set_netlist(circ.WahWah_test.S, circ.WahWah_test.V)
    c1.linearize(T(1), T(2), keep_dc=False)
    c1.remove_element("C2")
    c1.remove_element("R8")
    c1.join_net("V6", GND)
    c0 = Circuit(c1)
    c1.remove_element("C1")
    c1.join_net("V3", "V2")
    f0 = calc_highpass_f0(c0, c1)
    if linear:
        c1.create_faust_module(pre_filter="dcblockerat(%s)" % f0)
    else:
        c1.sig.timespan = 0.2
        #c1.grid_error_limit = 1e-1
        c1.input_amplitude = 3
        c1.pre_filter = highpass_as_prefilter(f0, c1.FS)
        c1.set_use(SIM_TABLE)
        c1.sim_c.set_variable("hotpotz", 1) # gives largest values for range calculation
        c1.deploy(".")

if __name__ == "__main__":
    set_log_level(INFO)
    build_module(44100, "", linear=True)
    build_module(44100, "_44")
    build_module(48000, "_48")
